import { Component, Inject, Input, OnInit, Optional } from '@angular/core';
import { FormGroup } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { NbDialogService, NbToastrService } from '@nebular/theme';
import { TranslateService } from '@ngx-translate/core';
import { DashboardPage, IDashboardPage } from '../../../model/dashboard-page.model';
import { DashboardPageCrudService } from '../services/dashboard-page-crud.service';
import { Location } from '@angular/common';
import { FormManagerService } from '../services/form-manager.service';
import { MenuType } from '../../../model/enumeration/menu-type.model';
import { MenuItemCrudService } from '../services/menu-item-crud.service';
import { Draggable, GridsterConfig, GridsterItem, PushDirections, Resizable } from 'angular-gridster2';
import { MAT_DIALOG_DATA } from '@angular/material/dialog';
import { ConfigService } from '@ngx-config/core';
import { DomSanitizer } from "@angular/platform-browser";
import * as L from 'leaflet';
import { DialogComponent } from '../../shared/dialog/dialog.component';
import { DashboardComponentDialogComponent } from '../dashboard-component-dialog/dashboard-component-dialog.component';
import { DashboardGuideComponent } from '../dashboard-guide/dashboard-guide.component';


interface Safe extends GridsterConfig {
  draggable: Draggable;
  resizable: Resizable;
  pushDirections: PushDirections;
}

interface DashboardModalData {
  parsedId: number;
  parsedEditMode: boolean;
}

@Component({
  selector: 'ngx-dashboard-page',
  templateUrl: './dashboard-page.component.html',
  styleUrls: ['./dashboard-page.component.scss']
})
export class DashboardPageComponent implements OnInit {
  @Input() page: FormGroup = new FormGroup({});
  public dashboardPage: DashboardPage = new DashboardPage();

  stepperIndex: number = 0;
  
  sharedType = MenuType.values().sort((x, y) => x.value.localeCompare(y.value));

  public options: GridsterConfig;
  public items: Array<GridsterItem>;
  public itemsClone: Array<GridsterItem>;

  public id: string;
  public componentType: string;
  public isEditable = true;
  pageId: string; //For Targets
  pageName: string; //For MenuItem
  disabledTarget: boolean; //For Targets
  private ENABLE_DELAY: boolean;
  private DELAY: number;

  currentSelected: string;
  currentShared: string;

  selected: string;

  linearMode = true;

  enableNextGridster = false;

  sharedDomain = [{ text: "Yes", val: true }, { text: "No", val: false }];

  constructor(
    private formService: FormManagerService,
    private menuItemService: MenuItemCrudService,
    private router: Router,
    private route: ActivatedRoute,
    private _sanitizer: DomSanitizer,
    private toastrService: NbToastrService,
    private translateService: TranslateService,
    private _location: Location,
    public dialogService: NbDialogService,
    public dashboardPageService: DashboardPageCrudService,
    private configService: ConfigService,
    @Optional() @Inject(MAT_DIALOG_DATA) public dashboardModalData: DashboardModalData
  ) {
    this.ENABLE_DELAY = this.configService.getSettings("dashboard-controller.enable_delay");
    this.DELAY = this.configService.getSettings("dashboard-controller.delay");
  }


  ngOnInit(): void {
    this.options = this.getDefaultOption();
    this.items = [];
    this.itemsClone = [];
    this.page = this.formService.pageFormCreator(new DashboardPage());
    this.route.paramMap.subscribe(paramMap => {
      this.items = [];
      this.id = paramMap.get('id');

      if (this.dashboardModalData != null && this.dashboardModalData != undefined) {
        this.id = this.dashboardModalData.parsedId as unknown as string;
        this.isEditable = this.dashboardModalData.parsedEditMode;
        this.options = this.getDefaultOption();
      }
      console.log("Retrieved Content for Page with Id: " + this.id);

      this.dashboardPageService.getPageById(this.id).subscribe((data) => {
        console.log(data);
        this.dashboardPage = data;
        if (data.content !== "" && data.content !== undefined && data.content !== null) {

          if (this.ENABLE_DELAY == true) {
            this.fillItemsWithDelay(data);
          } else {
            this.items = JSON.parse(data.content);
          }

        }
      });
      console.log("Content of DahsboardPage var");
      console.log(this.dashboardPage);
    });

    this.route.queryParams.subscribe(params => {
      if (params['isEditable'] != undefined && params['isEditable'] != null) {
        this.isEditable = params['isEditable'] == "true" ? true : false;
        this.options = this.getDefaultOption();
      }
    });
    console.log("Content of DahsboardPage var");
    console.log(this.dashboardPage);
  }


  dataType(): Array<string> {
    const keys = Object.keys(this.sharedType);
    return keys.slice(keys.length / 10, keys.length - 1);
  }

  changeEvent: DashboardPageComponent;

  handleStepChange(e: DashboardPageComponent): void {
    this.changeEvent = e;
  }


  private fillItemsWithDelay(data: IDashboardPage) {
    try {
      let itemsTemp = JSON.parse(data.content);
      itemsTemp.forEach((element, i) => {
        if (element['chart'] != undefined && element['chart'] != null) {
          setTimeout(() => {
            console.log("ELEMENT");
            console.log(element);
            this.items.push(element);
          }, i * this.DELAY);
        } else {
          this.items.push(element);
        }
      });
    } catch (e) {
      console.log("## Error in Dashboard page content during visualization due to " + e);
      this.toastrService.danger(this.translateService.instant('dashboardPage.visualization_catch_error'));
    }
  }


  removeItem(item) {
    this.dialogService.open(DialogComponent,
      {
        context: {
          title: this.translateService.instant('general.delete') + " " + this.translateService.instant('general.component'),
          message: this.translateService.instant('general.deleteConfirm')
        }
      })
      .onClose.subscribe(
        (res) => {
          this.items.splice(this.items.indexOf(item), 1);
        }, err => {
          // console.log(err);
          this.toastrService.danger(this.translateService.instant('dashboardPage.operation_failed'), this.translateService.instant('general.error'));
        }
      );
  }

  addItem() {
    let item: GridsterItem = { cols: 2, rows: 2, y: 0, x: 0 }; //default added item
    this.items.push(item);
  }

  editItem(item: GridsterItem, items: GridsterItem[]) {
    const dialogRef = this.dialogService
      .open(DashboardComponentDialogComponent, {
        context: {
          item,
          items,
        },
      })
      .onClose.subscribe(
        (item) => {
          item && (this.items = item) && this.options.api.optionsChanged() // optionsChanged() refreshes and applies changes
        }
      );
  }

  changedOptions(): void {
    if (this.options.api && this.options.api.optionsChanged) {
      this.options.api.optionsChanged();
    }
  }

  // saveDashboard(){
  //   this.dashboardPage.content =  JSON.stringify(this.items);
  //   this.dashboardPageService.updatePageById(this.dashboardPage)
  //   .subscribe((res) => {
  //     // console.log('## Saved Dashboard with id:' + this.dashboardPage.id);
  //     this.toastrService.success(this.translateService.instant('dashboardPage.updated'), this.translateService.instant('general.success'));
  //   },err => {
  //     // console.log(err);
  //     this.toastrService.danger(this.translateService.instant('dashboardPage.operation_failed'), this.translateService.instant('general.error'));
  //   });
  // }

  /**
   * Return modified string trusted (in case of base 64 string) or untouched image string in case or URL
   * @param item object with Image URL or image string in BASE 64
   * @returns trasted image url or string
   */
  buildImage(item: any) {
    if (item.url == false) {
      if (item.image != undefined && item.image.length > 10 && "data:image/jpeg;base64" == item.image.substr(0, 10)) {
        return this._sanitizer.bypassSecurityTrustResourceUrl(item.image);
      } else {
        return item.image;
      }
    } else {
      if (item.imageUrl != undefined && item.imageUrl.length > 10 && "data:image/jpeg;base64" == item.imageUrl.substr(0, 10)) {
        return this._sanitizer.bypassSecurityTrustResourceUrl(item.imageUrl);
      } else {
        return item.imageUrl;
      }
    }
  }

  private getDefaultOption(): any {
    return {
      pushItems: true,
      minCols: 20,
      minRows: 12,
      fixedRowHeight: 120,
      setGridSize: true,
      mobileBreakpoint: 0,
      gridType: "scrollVertical",
      resizable: {
        enabled: this.isEditable,
      },
      draggable: {
        enabled: this.isEditable,
      },
      margin: 10,
      outerMargin: true,
      swap: this.isEditable
    };
  }


  /** MAP Section WIP **/
  mapOptions = {
    layers: [
      L.tileLayer('http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', { maxZoom: 18, attribution: '...' }),
    ],
    zoom: 9,
    center: L.latLng({ lat: 52.372664783594274, lng: 4.8950958251953125 }),
  };


  onMapReady(map: L.Map, mapUrl: string) {
    console.log("### onMapReady, map ==> " + map);
    this.initializeMap(mapUrl, map);
  }

  public getMapLayer(title, l): L.GeoJSON {
    let layer = L.geoJSON(l,
      {
        onEachFeature: (feature, layer) => {
          let text = "";
          for (let p in feature.properties) {
            text += `${p}: ${feature.properties[p]}\n`;
          }
          layer.bindPopup(text);
        },
        style: function () {
          return { color: '#248175', weight: 7, opacity: 0.8 }
        }
      },
    ).on('popupclose', ($event) => { })
    layer['_id'] = title;
    return layer;
  }


  private initializeMap(mapUrl: string, map: L.Map) {
    this.dashboardPageService.getGeoJsonMap(mapUrl).subscribe(res => {
      let currentLayer = this.getMapLayer(mapUrl, <any>res);
      console.log("# initializeMap, currentLayer => " + currentLayer);
      setTimeout(() => {
        map.invalidateSize();
        map.addLayer(currentLayer);
        map.fitBounds(currentLayer.getBounds(), {
          padding: [50, 50]
        });
      }, 50);
      L.tileLayer('http://{s}.tile.osm.org/{z}/{x}/{y}.png', {
        attribution: '&copy; <a href="http://osm.org/copyright">OpenStreetMap</a> contributors'
      }).addTo(map);
    });
  }

  changeSelected(event) {
    this.currentSelected = event;
    console.log("CURRENT SELECTED");
    console.log(this.currentSelected);
  }

  changeCurrentSharedType(event) {
    this.currentShared = event;
    console.log("CURRENT changeCurrentSharedType");
    console.log(this.currentShared);
  }
  editPage(page: DashboardPage) {
    this.page = this.formService.pageFormCreator(page);
    this.pageName = page.name;
    this.menuItemService.getByPageId(this.pageId).subscribe(
        (data) => { this.disabledTarget = data!=null && data.length>0 ? data[0].type != MenuType.SHARED : true; }, 
        error => (this.showError(error)) 
      );
  }

  // isShared(): boolean{
  //   if(this.selected == "SHARED"){
  //     return false;
  //   }
  //   return true;
  // }

  // storePage(): void {
  //   if (this.page.value.id == null) {
  //     console.log('## CREATE DASHBOARD WITH ID:' + this.page.value.id);
  //     let tmp = <DashboardPage>this.page.getRawValue();
  //     this.pageName = tmp.name;
  //     this.service.createPage(tmp).subscribe(
  //       (data: DashboardPage) => {
  //         this.router.navigate(['/pages/dashboard-management/manage-dashboard-pages']);
  //         this.showSuccess();
  //       },
  //       error => (this.showError(error))
  //     );

  //   }
  //   else {
  //     console.log('# MODIFY DASHBOARD WITH ID:' + this.page.value.id);
  //     let tmp = <DashboardPage>this.page.getRawValue();
  //     this.pageName = tmp.name;
  //     this.service.updatePageById(tmp).subscribe(
  //       (data: DashboardPage) => {
  //         this.editPage(data);
  //         this.showSuccess();
  //       },
  //       error => (this.showError(error))
  //     );

  //   }
  // }

  createDashboard() {
    this.stepperIndex++;
    let dashboardPage = <DashboardPage>this.page.getRawValue();
    this.disabledTarget = dashboardPage.type != undefined ? dashboardPage.type != MenuType.SHARED : true;
    this.dashboardPageService.createPage(dashboardPage).subscribe(
      (data: DashboardPage) => {
        data.content = JSON.stringify(this.items);
        this.dashboardPageService.updatePageById(data)
          .subscribe((res) => {
            this.pageId = res.id.toString();
            this.showSuccess();
            if(this.selected!=='SHARED'){
              this.router.navigate(['/pages/dashboard-management/manage-dashboard-pages'])
            }
          })
      },
      error => (this.showError(error))
    );
  }

  showSuccess() {
    this.page.value.id == null
      ? this.toastrService.success(this.translateService.instant('dashboardPage.created'), this.translateService.instant('Success'))
      : this.toastrService.success(this.translateService.instant('dashboardPage.updated'), this.translateService.instant('Success'));
  }

  showError(error) {
    this.toastrService.danger(error.error.message, this.translateService.instant('general.error'))
  }



  // updateSharedPersonalTypology($event) {    
  //   this.menuItemService.getByPageId(this.pageId).subscribe(
  //     (data) => {
  //     this.disabledTarget = data!=null && data.length>0 ? data[0].type != MenuType.SHARED : true;
  //     if (this.disabledTarget){
  //       this.page.get("shared").disable();
  //     }else{
  //       this.page.get("shared").enable();  
  //     }
  //   },
  //   error => (this.showError(error))
  //   );
  // }

  getMessageDisabledTarget() {
    return this.disabledTarget
      ? this.translateService.instant('dashboardPage.not_mandatory_field')
      : this.translateService.instant('general.mandatory_field');
  }

  openGuide(){
    const dialogRef = this.dialogService
    .open(DashboardGuideComponent,
      { context: { info: "create dashboard", stepperIndex: this.stepperIndex} });
  }
  
  increaseStep(){
    this.stepperIndex++
  }

  decreaseStep(){
    this.stepperIndex--;
  }

}
