import { Component, Inject, OnInit, Optional } from "@angular/core";
import { ActivatedRoute, Router } from "@angular/router";
import { NbDialogService, NbToastrService } from "@nebular/theme";
import { Draggable, GridsterConfig, GridsterItem, PushDirections, Resizable } from "angular-gridster2";
import { DashboardPage, IDashboardPage } from "../../../model/dashboard-page.model";
import { DashboardComponentDialogComponent } from "../dashboard-component-dialog/dashboard-component-dialog.component";
import { DashboardPageCrudService } from "../services/dashboard-page-crud.service";
import { TranslateService } from "@ngx-translate/core";
import { DialogComponent } from "../../shared/dialog/dialog.component";
import { Location } from '@angular/common';
import { DomSanitizer } from "@angular/platform-browser";
import * as L from 'leaflet';
import { MAT_DIALOG_DATA } from "@angular/material/dialog";
import { ConfigService } from '@ngx-config/core';
import { OidcUserInformationService } from "../../../auth/services/oidc-user-information.service";

interface Safe extends GridsterConfig {
  draggable: Draggable;
  resizable: Resizable;
  pushDirections: PushDirections;
}

interface DashboardModalData {
  parsedId: number;
  parsedEditMode: boolean;
  dashboardPage: DashboardPage;
}

@Component({
  selector: 'ngx-manage-dashboard-page-content',
  templateUrl: './manage-dashboard-page-content.component.html',
  styleUrls: ['./manage-dashboard-page-content.component.scss']
})
export class ManageDashboardPageContentComponent implements OnInit {
  public options: GridsterConfig;
  public items: Array<GridsterItem>;
  public itemsClone: Array<GridsterItem>;
  public dashboardPage: DashboardPage = new DashboardPage();
  public id: string;
  public componentType: string;
  public isEditable = true;  //Change options to disable drag and drop
  public userRole: string;
  //WorkAroud for BUG on ng-gridster+iframe
  private ENABLE_DELAY: boolean;
  private DELAY: number;

  constructor(public dialogService: NbDialogService,
    private router: Router,
    public dashboardPageService: DashboardPageCrudService,
    private route: ActivatedRoute,
    private _sanitizer: DomSanitizer,
    private toastrService: NbToastrService,
    private translateService: TranslateService,
    private _location: Location,
    private configService: ConfigService, private userService: OidcUserInformationService,
    @Optional() @Inject(MAT_DIALOG_DATA) public dashboardModalData: DashboardModalData
  ) {
    this.ENABLE_DELAY = this.configService.getSettings("dashboard-controller.enable_delay");
    this.DELAY = this.configService.getSettings("dashboard-controller.delay");
  }

  ngOnInit() {
    this.options = this.getDefaultOption();
    this.items = [];
    this.itemsClone = [];

    this.route.paramMap.subscribe(paramMap => {
      this.items = [];
      this.id = paramMap.get('id');
      if (this.dashboardModalData != null && this.dashboardModalData != undefined) {
        this.id = this.dashboardModalData.parsedId as unknown as string;
        this.isEditable = this.dashboardModalData.parsedEditMode;
        this.options = this.getDefaultOption();
      }
      //console.log("Retrieved Content for Page with Id: " +  this.id);
      console.log("#GET USER ROLE");
      this.userService.getRole().subscribe(res=> {
        this.userRole = res[0];
      });
      if(this.userRole !== "ANONYMOUS"){
        this.dashboardPageService.getPageById(this.id).subscribe((data) => {
          this.dashboardPage = data;
          if (data.content !== "" && data.content !== undefined && data.content !== null) {
  
            if (this.ENABLE_DELAY == true) {
              this.fillItemsWithDelay(data);
            } else {
              this.items = JSON.parse(data.content);
            }
  
          }
        });  
      } else{
        this.fillItemsWithDelay(this.dashboardModalData.dashboardPage);
      }

    });

    this.route.queryParams.subscribe(params => {
      if (params['isEditable'] != undefined && params['isEditable'] != null) {
        this.isEditable = params['isEditable'] == "true" ? true : false;
        this.options = this.getDefaultOption();
      }
    });

  }

  private fillItemsWithDelay(data: IDashboardPage) {
    try {
      let itemsTemp = JSON.parse(data.content);
      itemsTemp.forEach((element, i) => {
        if (element['chart'] != undefined && element['chart'] != null) {
          setTimeout(() => {
            // console.log(element);
            this.items.push(element);
          }, i * this.DELAY);
        } else {
          this.items.push(element);
        }
      });
    } catch (e) {
      console.log("## Error in Dashboard page content during visualization due to " + e);
      this.toastrService.danger(this.translateService.instant('dashboardPage.visualization_catch_error'));
    }
  }


  removeItem(item) {
    this.dialogService.open(DialogComponent,
      {
        context: {
          title: this.translateService.instant('general.delete') + " " + this.translateService.instant('general.component'),
          message: this.translateService.instant('general.deleteConfirm')
        }
      })
      .onClose.subscribe(
        (res) => {
          this.items.splice(this.items.indexOf(item), 1);
        }, err => {
          // console.log(err);
          this.toastrService.danger(this.translateService.instant('dashboardPage.operation_failed'), this.translateService.instant('general.error'));
        }
      );
  }

  addItem() {
    let item: GridsterItem = { cols: 2, rows: 2, y: 0, x: 0 }; //default added item
    this.items.push(item);
  }

  editItem(item: GridsterItem, items: GridsterItem[]) {
    const dialogRef = this.dialogService
      .open(DashboardComponentDialogComponent, {
        context: {
          item,
          items,
        },
      })
      .onClose.subscribe(
        (item) => {
          item && (this.items = item) && this.options.api.optionsChanged() // optionsChanged() refreshes and applies changes
        }
      );
  }

  changedOptions(): void {
    if (this.options.api && this.options.api.optionsChanged) {
      this.options.api.optionsChanged();
    }
  }

  saveDashboard() {
    this.dashboardPage.content = JSON.stringify(this.items);
    this.dashboardPageService.updatePageById(this.dashboardPage)
      .subscribe((res) => {
        // console.log('## Saved Dashboard with id:' + this.dashboardPage.id);
        this.toastrService.success(this.translateService.instant('dashboardPage.updated'), this.translateService.instant('general.success'));
        this.router.navigate(['/pages/dashboard-management/manage-dashboard-pages']);
      }, err => {
        // console.log(err);
        this.toastrService.danger(this.translateService.instant('dashboardPage.operation_failed'), this.translateService.instant('general.error'));
      });
  }


  /** Ask confirm before to go back */
  backClicked() {
    this.dialogService.open(DialogComponent,
      {
        context: {
          title: this.translateService.instant('dashboardComponent.title'),
          message: this.translateService.instant('dashboardPage.back_modal')
        }
      })
      .onClose.subscribe(res => {
        if (res) {
          this._location.back();
        } else return;
      });
  }

  /**
   * Return modified string trusted (in case of base 64 string) or untouched image string in case or URL
   * @param item object with Image URL or image string in BASE 64
   * @returns trasted image url or string
   */
  buildImage(item: any) {
    if (item.url == false) {
      if (item.image != undefined && item.image.length > 10 && "data:image/jpeg;base64" == item.image.substr(0, 10)) {
        return this._sanitizer.bypassSecurityTrustResourceUrl(item.image);
      } else {
        return item.image;
      }
    } else {
      if (item.imageUrl != undefined && item.imageUrl.length > 10 && "data:image/jpeg;base64" == item.imageUrl.substr(0, 10)) {
        return this._sanitizer.bypassSecurityTrustResourceUrl(item.imageUrl);
      } else {
        return item.imageUrl;
      }
    }
  }

  private getDefaultOption(): any {
    return {
      pushItems: true,
      minCols: 20,
      minRows: 12,
      fixedRowHeight: 120,
      setGridSize: true,
      mobileBreakpoint: 0,
      gridType: "scrollVertical",
      resizable: {
        enabled: this.isEditable,
      },
      draggable: {
        enabled: this.isEditable,
      },
      margin: 10,
      outerMargin: true,
      swap: this.isEditable
    };
  }


  /** MAP Section WIP **/
  mapOptions = {
    layers: [
      L.tileLayer('http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', { maxZoom: 18, attribution: '...' }),
    ],
    zoom: 9,
    center: L.latLng({ lat: 52.372664783594274, lng: 4.8950958251953125 }),
  };


  onMapReady(map: L.Map, mapUrl: string) {
    console.log("### onMapReady, map ==> " + map);
    this.initializeMap(mapUrl, map);
  }

  public getMapLayer(title, l): L.GeoJSON {
    let layer = L.geoJSON(l,
      {
        onEachFeature: (feature, layer) => {
          let text = "";
          for (let p in feature.properties) {
            text += `${p}: ${feature.properties[p]}\n`;
          }
          layer.bindPopup(text);
        },
        style: function () {
          return { color: '#248175', weight: 7, opacity: 0.8 }
        }
      },
    ).on('popupclose', ($event) => { })
    layer['_id'] = title;
    return layer;
  }


  private initializeMap(mapUrl: string, map: L.Map) {
    this.dashboardPageService.getGeoJsonMap(mapUrl).subscribe(res => {
      let currentLayer = this.getMapLayer(mapUrl, <any>res);
      console.log("# initializeMap, currentLayer => " + currentLayer);
      setTimeout(() => {
        map.invalidateSize();
        map.addLayer(currentLayer);
        map.fitBounds(currentLayer.getBounds(), {
          padding: [50, 50]
        });
      }, 50);
      L.tileLayer('http://{s}.tile.osm.org/{z}/{x}/{y}.png', {
        attribution: '&copy; <a href="http://osm.org/copyright">OpenStreetMap</a> contributors'
      }).addTo(map);
    });
  }


}
