import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { NbDialogService, NbToastrService } from '@nebular/theme';
import { TranslateService } from '@ngx-translate/core';
import { LocalDataSource } from 'ng2-smart-table';
import { DashboardPage } from '../../../model/dashboard-page.model';
import { MenuType } from '../../../model/enumeration/menu-type.model';
import { IMenuBlock } from '../../../model/menu-block.model';
import { IMenuItem, MenuItem } from '../../../model/menu-item.model';
import { DialogComponent } from '../../shared/dialog/dialog.component';
import { TableActionsComponent } from '../../shared/table-actions/table-actions.component';
import { MenuBlockCrudService } from '../services/menu-block-crud.service';
import { MenuItemCrudService } from '../services/menu-item-crud.service';

@Component({
  selector: 'ngx-menu-filter',
  templateUrl: './menu-filter.component.html',
  styleUrls: ['./menu-filter.component.scss']
})
export class MenuFilterComponent implements OnInit {
  @Input() pageId: string;
  @Input() pageName: string;
  @Output() sendFilter = new EventEmitter<any>() //Broadcast event for disabledTarget
  
  //SMART TABLES
  source: LocalDataSource;
  settings: any;
  sharedMenuItems: Array<IMenuItem>=[];

  toAddMenuItem: IMenuItem;
  fullListBlockEditable: Array<IMenuBlock> = [];
  fullListBlockShared: Array<IMenuBlock> = [];
  fullListBlockPersonal: Array<IMenuBlock> = [];
  fullListBlockPublic: Array<IMenuBlock> = [];

  form: FormGroup;
  selectedMenuBlock: IMenuBlock;
  selectedMenuItemLabel: string;

  constructor(
    private fb: FormBuilder,
    private menuItemService: MenuItemCrudService,
    private menuBlockCrudService: MenuBlockCrudService,
    private dialogService: NbDialogService,
    private toastrService: NbToastrService,
    private translateService: TranslateService) { }

  ngOnInit(): void {
    this.initTableSettings();
    this.source = new LocalDataSource();
    this.fetchTableData();

    this.form = this.fb.group({
      selectedMenuBlock: [this.selectedMenuBlock, Validators.nullValidator],
      selectedMenuItemLabel: [this.selectedMenuItemLabel, Validators.nullValidator],
    })

    this.menuBlockCrudService.getAllEditable().subscribe(res => {
      this.fullListBlockEditable = res;
      this.fullListBlockShared = this.fullListBlockEditable.filter(x => x.type == MenuType.SHARED);
      this.fullListBlockPersonal = this.fullListBlockEditable.filter(x => x.type == MenuType.PRIVATE);
      this.fullListBlockPublic = this.fullListBlockEditable.filter(x => x.type == MenuType.PUBLIC);
    });
    
  }

  fetchTableData() {
    if (this.pageId!=undefined) {
      this.menuItemService.getByPageId(this.pageId).subscribe(
        //Success
        (data: Array<IMenuItem>) => {
          this.sharedMenuItems = data;
          for(let currMenuItem of this.sharedMenuItems){
            // Read MenuItem Label
            this.menuBlockCrudService.getById(''+currMenuItem.menuBlock.id).subscribe(res2 => {
              if (res2 != undefined && res2 != null && res2.label != undefined && res2.label != null) {
                this.sharedMenuItems[0].menuBlock.label = res2.label; //Only a Menu Block is linked to item!!
                this.source.load(this.sharedMenuItems);
              } else {
                this.source.load(this.sharedMenuItems);
              }
            });
          }
          if (this.sharedMenuItems){
            this.source.load(this.sharedMenuItems);
          }
        },
        //Error
        error => (this.showError(error)));
    }
  }

  addMenuItem() {
    console.log("# Add MenuItem linked to dashboard with pageId =>" + this.pageId);
    const menuBlock = this.form.get('selectedMenuBlock').value;
    const menuItemLabel = this.form.get('selectedMenuItemLabel').value;
    
    if (menuBlock == undefined || menuBlock == null){
      this.showWarning('menuBlock.select_menu_block');
      return;
    }

    if (menuItemLabel == undefined || menuItemLabel == null){
      this.showWarning('menuItem.add_menu_item_label');
      return;
    }
    
    this.toAddMenuItem = this.buildMenuItem(menuBlock, menuItemLabel, menuBlock.type, Number(this.pageId));
    this.menuItemService.create(this.toAddMenuItem).subscribe(
      (data: MenuItem) => {
        this.showSuccess('menuItem.created');
        console.log(data)
        this.fetchTableData();
        this.form.reset(); //Clean select and input

        this.broadcastInfo(); //Broadcast event for disabledTarget
      },
      error => (this.showError(error))
    );
  }

  private buildMenuItem(menuBlock: IMenuBlock, menuItemLabel: string, type:MenuType, pageId: number) {
    const newItem = new MenuItem();
    newItem.dashboardPage = new DashboardPage();
    newItem.dashboardPage.id = pageId;
    newItem.type = type;
    newItem.label = menuItemLabel;
    newItem.code = menuItemLabel!=null ? menuItemLabel.replace(/\s/g, "_").toUpperCase() : menuItemLabel;
    newItem.menuBlock = menuBlock;
    return newItem;
  }

  showSuccess(operation:string) {
    this.toastrService.success(this.translateService.instant(operation), this.translateService.instant('general.success'));
  }
  
  showError(error) {
    this.toastrService.danger(error.error.message, this.translateService.instant('general.error'))
  }

  showWarning(operation:string) {
    this.toastrService.warning(this.translateService.instant(operation), this.translateService.instant('general.warning'));
  }


  initTableSettings(): void {
    let menuTypeFilter=[];
    for(let mType of MenuType.values()){
      menuTypeFilter.push({ value: mType.enumVal, title: this.translateService.instant("MenuType."+mType.enumVal) })
    }

    this.settings = {
      mode: 'external',
      attr: {
        class: 'table table-bordered'
      },
      columns: {
        /*id: {
          title: this.translateService.instant('menuItem.id'),
        },
        */
        type: {
          title: this.translateService.instant('menuItem.type'),
          valuePrepareFunction: (cell, row) => { 
            if(row.type!=undefined && row.type!=null)
              return this.translateService.instant('MenuType.' + row.type) 
            else return '';
          },
          filter: {
            type: 'list',
            config: {
              selectText: '',
              list: menuTypeFilter
            },
          }
        },
        menuBlock: {
          title: this.translateService.instant('menuBlock.title') + " " + this.translateService.instant('menuBlock.label'),
          valuePrepareFunction: (cell, row) => { 
            if(row.menuBlock!=undefined && row.menuBlock!=null){
              return row.menuBlock.label;
            }
            else return '';
          }
        },
        /*code: {
          title: this.translateService.instant('menuItem.code'),
        },
        */
        label: {
          title: this.translateService.instant('menuItem.title') + " " + this.translateService.instant('menuItem.label')
        },
        actions: {
          title: this.translateService.instant('general.actions'),
          filter: false,
          sort: false,
          type: 'custom',
          valuePrepareFunction: (cell, row) => {
            return [
              {action:'delete', enabled:true, status:"danger", icon:"trash-2", tooltip:this.translateService.instant('general.delete')+ " " + this.translateService.instant('menuItem.title')}
            ];
          }, 
          renderComponent: TableActionsComponent,
          onComponentInitFunction: (instance) => {
            instance.onCustom.subscribe(value => {
              this.onCustom(value,instance.rowData);
            });
          },          
        }

      },

      actions: {
        add: false,
        edit: false,
        delete: false,
      },

    };
  }

  onCustom(event, data) {
    if (event == 'delete') {
      this.dialogService
          .open( DialogComponent, { context: { 
              title: this.translateService.instant('general.delete') + " " + this.translateService.instant('menuItem.title') , 
              message: this.translateService.instant('general.deleteConfirm')} })
          .onClose.subscribe(res => {
        if (res) {
          this.menuItemService.deleteById(data.id).subscribe( (res) => {
            this.fetchTableData();
            console.log(`## MenuItem with id=${data.id} has been deleted.`),
            this.showSuccess('menuItem.deleted');

            this.broadcastInfo(); //Broadcast event for disabledTarget
          },
            error => (this.showError(error))
          );
        } else return;

      });
    };
  }

  disableButton() {
    return this.pageId == null || (this.sharedMenuItems!=undefined && this.sharedMenuItems.length > 0) ? true : false;
  }

  generateLabel(){
    this.form.patchValue( { selectedMenuItemLabel: this.pageName });  
  }

  broadcastInfo() {
    this.sendFilter.emit(this.form.getRawValue());
  }

}
