import { Component, Input, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { NbDialogService, NbToastrService } from '@nebular/theme';
import { ConfigService } from '@ngx-config/core';
import { TranslateService } from '@ngx-translate/core';
import { LocalDataSource } from 'ng2-smart-table';
import { DashboardPage } from '../../../model/dashboard-page.model';
import { TargetType } from '../../../model/enumeration/target-type.model';
import { IGroupRepresentation } from '../../../model/group-representation.model';
import { IRoleRepresentation } from '../../../model/role-representation.model';
import { ITargetAvailable } from '../../../model/target-available.model';
import { ITarget, Target } from '../../../model/target.model';
import { IUser } from '../../../model/user.model';
import { DialogComponent } from '../../shared/dialog/dialog.component';
import { TableActionsComponent } from '../../shared/table-actions/table-actions.component';
import { TargetAvailableCrudService } from '../services/target-available-crud.service';
import { TargetCrudService } from '../services/target-crud.service';
import { UserCrudService } from '../services/user-crud.service';

@Component({
  selector: 'ngx-target-filter',
  templateUrl: './target-filter.component.html',
  styleUrls: ['./target-filter.component.scss']
})
export class TargetFilterComponent implements OnInit {
  @Input() pageId: string;
  @Input() disabledTarget: boolean = false;  // USED TO HIDE TARGET

  //SMART TABLES
  source: LocalDataSource;
  settings: any;
  sharedTargets: Array<ITarget>=[];

  form:FormGroup;
  toAddTarget: ITarget;  

  //Available Targets
  useIDM4Target: boolean;

  //(option B) Uses data from internal table (TARGET_AVAILABLE)
  fullListAvailablePerson:Array<ITargetAvailable>=[];
  fullListAvailableRole:Array<ITargetAvailable>=[];
  fullListAvailableGroup:Array<ITargetAvailable>=[];
  selectedAvailablePerson: ITargetAvailable;
  selectedAvailableRole: ITargetAvailable;
  selectedAvailableGroup: ITargetAvailable;

  //(option A) Uses data from IDM
  fullListAvailablePersonIDM:Array<IUser>=[];
  fullListAvailableRoleIDM:Array<IRoleRepresentation>=[];
  fullListAvailableGroupsIDM:Array<IGroupRepresentation>=[];
  selectedAvailablePersonIDM: IUser;
  selectedAvailableRoleIDM: IRoleRepresentation;
  selectedAvailableGroupIDM: IGroupRepresentation;
  constructor(
      private fb: FormBuilder,
      private targetService: TargetCrudService, 
      private configService: ConfigService,
      private targetAvailableCrudService: TargetAvailableCrudService, 
      private userCrudService: UserCrudService, 
      private dialogService: NbDialogService, 
      private toastrService: NbToastrService,
      private translateService: TranslateService) 
  { }

  ngOnInit(): void {
    
    // CHANGE THIS TO READS USERS/ROLES (GROUPS) FROM IDM or INTERNAL TABLE:
    // true  => (option A) IDM
    // false => (option B) INTERNAL TABLE (DEMO)
    this.useIDM4Target = this.configService.getSettings("dashboard-controller.useIDM4Target");

    this.initTableSettings();
    this.source = new LocalDataSource();
    this.fetchTargetTableData();

    this.form = this.fb.group({
      selectedAvailablePerson: [this.selectedAvailablePerson, Validators.nullValidator],
      selectedAvailableRole: [this.selectedAvailableRole, Validators.nullValidator],
      selectedAvailableGroup: [this.selectedAvailableGroup, Validators.nullValidator],
      selectedAvailablePersonIDM: [this.selectedAvailablePersonIDM, Validators.nullValidator],
      selectedAvailableRoleIDM: [this.selectedAvailableRoleIDM, Validators.nullValidator],
      selectedAvailableGroupIDM: [this.selectedAvailableGroupIDM, Validators.nullValidator],
    });

    if (this.useIDM4Target == true) {
        //Retrieve list of Person/Roles/Groups from IDM
        this.userCrudService.getIdmUsers().subscribe( res =>{
          this.fullListAvailablePersonIDM = res;
        });
        this.userCrudService.getIdmRoles().subscribe( res =>{
          this.fullListAvailableRoleIDM = res;
        });
        this.userCrudService.getIdmGroups().subscribe( res =>{
          this.fullListAvailableGroupsIDM = res;
        });
    } else {
        //Retrieve list of Person/Role/Groups from internal table
        this.targetAvailableCrudService.getAvailableTargetsByType(TargetType.PERSON).subscribe( res =>{
          this.fullListAvailablePerson = res;
        });
        this.targetAvailableCrudService.getAvailableTargetsByType(TargetType.ROLE).subscribe( res =>{
          this.fullListAvailableRole = res;
        });
        this.targetAvailableCrudService.getAvailableTargetsByType(TargetType.GROUP).subscribe( res =>{
          this.fullListAvailableGroup = res;
        });
    }

  }

  fetchTargetTableData() {
    //Retrieve Targets of the selected PageId
    if (this.pageId!=undefined) {
      this.targetService.getTargetsByPageId(this.pageId).subscribe(data => {
        this.sharedTargets=data;
        this.source.load(data);
      });
    }

  }

  addTargetPerson() {
    console.log("# Add Person linked to dashboard with pageId =>" + this.pageId);
    const person = this.useIDM4Target
                 ? this.form.get('selectedAvailablePersonIDM').value
                 : this.form.get('selectedAvailablePerson').value;
    console.log("CONST PERSON");
    console.log(person);
    if (person == undefined || person == null){
      this.showWarning('target.fill_person');
      return;
    }

    this.toAddTarget = this.useIDM4Target
                     ? this.buildUserTargetIDM(person, TargetType.PERSON, Number(this.pageId))
                     : this.buildTarget(person, TargetType.PERSON, Number(this.pageId));

    this.targetService.createTarget(this.toAddTarget).subscribe(
      (data: Target) => {
        console.log("DATA IN TARGET SERVICE");
        console.log(data);
        this.showSuccess('target.created');
        this.fetchTargetTableData();
        this.form.reset(); //Clean select person
      },
      error => {console.log("THERE IS AN ERROR"); this.showError(error)}
    );
  }

  addTargetRole() {
    console.log("# Add Role linked to dashboard with pageId =>" + this.pageId);
    const role = this.useIDM4Target
               ? this.form.get('selectedAvailableRoleIDM').value
               : this.form.get('selectedAvailableRole').value;
    
    if (role == undefined || role == null){
      this.showWarning('target.fill_role');
      return;
    }

    this.toAddTarget = this.useIDM4Target
                     ? this.buildRoleTargetIDM(role, TargetType.ROLE, Number(this.pageId))
                     : this.buildTarget(role, TargetType.ROLE, Number(this.pageId));

    this.targetService.createTarget(this.toAddTarget).subscribe(
      (data: Target) => {
        this.showSuccess('target.created');
        this.fetchTargetTableData();
        this.form.reset(); //Clean select person
      },
      error => (this.showError(error))
    );
  }

  addTargetGroup() {
    console.log("# Add Group linked to dashboard with pageId =>" + this.pageId);
    const group = this.useIDM4Target
               ? this.form.get('selectedAvailableGroupIDM').value
               : this.form.get('selectedAvailableGroup').value;
    
    if (group == undefined || group == null){
      this.showWarning('target.fill_group');
      return;
    }

    this.toAddTarget = this.useIDM4Target
                     ? this.buildGroupTargetIDM(group, TargetType.GROUP, Number(this.pageId))
                     : this.buildTarget(group, TargetType.GROUP, Number(this.pageId));

    this.targetService.createTarget(this.toAddTarget).subscribe(
      (data: Target) => {
        this.showSuccess('target.created');
        this.fetchTargetTableData();
        this.form.reset(); //Clean select person
      },
      error => (this.showError(error))
    );
  }

  private buildTarget(target: ITargetAvailable, type: TargetType, pageId: number) {
    const curTarget = new Target();
    curTarget.dashboardPage = new DashboardPage();
    curTarget.dashboardPage.id = pageId;
    curTarget.type = type;
    curTarget.name = target.code;
    curTarget.note = target.value;
    return curTarget;
  }

  private buildUserTargetIDM(person: IUser, type: TargetType, pageId: number) {
    const curTarget = new Target();
    curTarget.dashboardPage = new DashboardPage();
    curTarget.dashboardPage.id = pageId;
    curTarget.type = type;
    curTarget.name = person.username;
    //curTarget.note = person.firstName + " " + person.lastName + "("+person.email+")";
    curTarget.note = person.username;
    return curTarget;
  }

  private buildRoleTargetIDM(role: IRoleRepresentation, type: TargetType, pageId: number) {
    const curTarget = new Target();
    curTarget.dashboardPage = new DashboardPage();
    curTarget.dashboardPage.id = pageId;
    curTarget.type = type;
    curTarget.name = role.name;
    //curTarget.note = role.description;
    curTarget.note = role.name;
    return curTarget;
  }

  private buildGroupTargetIDM(group: IGroupRepresentation, type: TargetType, pageId: number) {
    const curTarget = new Target();
    curTarget.dashboardPage = new DashboardPage();
    curTarget.dashboardPage.id = pageId;
    curTarget.type = type;
    curTarget.name = group.name;
    curTarget.note = group.name;
    return curTarget;
  }

  initTableSettings(): void {
    let targetTypeFilter=[];
    for(let targetType of TargetType.values()){
      targetTypeFilter.push({ value: targetType.enumVal, title: this.translateService.instant("TargetType."+targetType.enumVal) })
    }

    this.settings = {
      mode: 'external',
      attr: {
        class: 'table table-bordered'
      },
      columns: {
        /*id: {
          title: this.translateService.instant('target.id'),
        },*/
        type: {
          title: this.translateService.instant('target.type'),
          valuePrepareFunction: (cell, row) => { 
            if(row.type!=undefined && row.type!=null)
              return this.translateService.instant('TargetType.' + row.type) 
            else return '';
          },
          filter: {
            type: 'list',
            config: {
              selectText: '',
              list: targetTypeFilter
            },
          }


        },
        name: {
          title: this.translateService.instant('target.name'),
        },
        note: {
          title: this.translateService.instant('target.value')
        },
        /*
        dashboardPage: {
          title: this.translateService.instant('target.dashboardPage'),
        }, */
        
        actions: {
          title: this.translateService.instant('general.actions'),
          filter: false,
          sort: false,
          type: 'custom',
          valuePrepareFunction: (cell, row) => {
            return [
              {action:'delete', enabled:true, status:"danger", icon:"trash-2", tooltip:this.translateService.instant('general.delete')+" " + this.translateService.instant('target.title')}
            ];
          }, 
          renderComponent: TableActionsComponent,
          onComponentInitFunction: (instance) => {
            instance.onCustom.subscribe(value => {
              this.onCustom(value,instance.rowData);
            });
          },          
        }

      },

      actions: {
        add: false,
        edit: false,
        delete: false,
      },

    };
  }


  onCustom(event, data) {
    if (event == 'delete') {
      this.dialogService
          .open( DialogComponent, { context: { 
              title: this.translateService.instant('general.delete') + " " + this.translateService.instant('general.targets') , message: this.translateService.instant('general.deleteConfirm') } })
          .onClose.subscribe(res => {
        if (res) {
          this.targetService.deleteTargetById(data.id).subscribe(() => {
            this.fetchTargetTableData();
            console.log(`# Target with id=${data.id} has been deleted.`),
              this.showSuccess('target.deleted');
          },
            error => (this.showError(error))
          );
        } else return;

      });
    };
  }

  showSuccess(operation:string) {
    this.toastrService.success(this.translateService.instant(operation), this.translateService.instant('general.success'));
  }
  
  showError(error) {
    this.toastrService.danger(error.error.message, this.translateService.instant('general.error'))
  }

  showWarning(operation:string) {
    this.toastrService.warning(this.translateService.instant(operation), this.translateService.instant('general.warning'));
  }

  /**
   * Filter to exclude from Available Person List the ones that are into the Target list
   * @param persons Array<IUser>
   * @returns Array<IUser>
   */
  filterIDMPersons(persons: Array<IUser>) {
    let result :Array<IUser> = [];
    if ( this.sharedTargets != undefined &&  this.sharedTargets.length>0 && persons != undefined &&  persons.length>0){
      persons.forEach(person => {
        let found = this.sharedTargets.filter(x=> x.name==person.username).map(x=>x.name);
        if ( found.length == 0 ) { result.push(person) }
      });
    } else { result = persons; }
    return result;
  }

  /**
   * Filter to exclude from Available Groups List the ones that are into the Target list
   * @param groups Array<IGroupRepresentation>
   * @returns Array<IGroupRepresentation>
   */
  filterIDMGroups(groups: Array<IGroupRepresentation>) {
    let result :Array<IGroupRepresentation> = [];
    if ( this.sharedTargets != undefined &&  this.sharedTargets.length>0 && groups != undefined &&  groups.length>0){
      groups.forEach(group => {
        let found = this.sharedTargets.filter(x=> x.name==group.name).map(x=>x.name);
        if ( found.length == 0 ) { result.push(group) }
      });
    } else { result = groups; }
    return result;
  }

 /**
  * Filter to exclude from Available Roles List the ones that are into the Target list
  * @param roles Array<IRoleRepresentation>
  * @returns Array<IRoleRepresentation>
  */
  filterIDMRoles(roles: Array<IRoleRepresentation>) {
    let result :Array<IRoleRepresentation> = [];
    if ( this.sharedTargets != undefined &&  this.sharedTargets.length>0 && roles != undefined &&  roles.length>0){
      roles.forEach(role => {
        let found = this.sharedTargets.filter(x=> x.name==role.name).map(x=>x.name);
        if ( found.length == 0 ) { result.push(role) }
      });
    } else { result = roles; }
    return result;
  }

  filterTarget(avTarget: Array<ITargetAvailable>, targetType: string) {
    let result :Array<ITargetAvailable> = [];
    const selectedType = (targetType==TargetType.GROUP ? TargetType.GROUP : (targetType==TargetType.ROLE ? TargetType.ROLE : TargetType.PERSON) );
    if ( this.sharedTargets != undefined && this.sharedTargets.length>0 && avTarget != undefined &&  avTarget.length>0){
      avTarget.forEach(curTarget => {
        let found = this.sharedTargets.filter(x=> x.name==curTarget.code && curTarget.type == selectedType).map(x=>x.name);
        if ( found.length == 0 ) { result.push(curTarget) }
      });
    } else { result = avTarget; }
    return result;
  }

}
