import { Component, OnInit } from '@angular/core';
import * as L from 'leaflet';
import 'leaflet.heat/dist/leaflet-heat.js';
import { MessinaClientService } from '../service/messina-client.service';

@Component({
  selector: 'ngx-single-map',
  styleUrls: ['./heatmap.component.scss'],
  template: `
   <div class="row d-flex justify-content-between align-items-center">
        <nb-card class="col-6" [nbSpinner]="loading">
            <nb-card-header>
              Messina LPT Critical Areas
              <ngx-heatmap-filter (sendFilter)="updateMap1($event)" [selected]="selectedPeriod1"></ngx-heatmap-filter>
            </nb-card-header>
            <nb-card-body>
              <div id="map1" #map1 class="map" 
                leaflet [leafletOptions]="options1" 
                [leafletLayers]="layers1"
                [leafletLayersControl]="layersControl1"
                (leafletMapReady)="onMapReady1($event)">
              </div>
            </nb-card-body>
            <nb-card-footer>
            Identification of <b>areas</b> where vehicles of public transportation are <b>stationary</b> for a certain time in a specific <b>period</b>.
            </nb-card-footer>
        </nb-card>
        <nb-card class="col-6" [nbSpinner]="loading">
            <nb-card-header>
              Messina LPT Critical Areas
              <ngx-heatmap-filter (sendFilter)="updateMap2($event)" [selected]="selectedPeriod2"></ngx-heatmap-filter>
            </nb-card-header>
            <nb-card-body>
              <div id="map2" #map2 class="map" 
                leaflet [leafletOptions]="options2" 
                [leafletLayers]="layers2"
                [leafletLayersControl]="layersControl2"
                (leafletMapReady)="onMapReady2($event)">
              </div>
            </nb-card-body>
            <nb-card-footer>
            Identification of <b>areas</b> where vehicles of public transportation are <b>stationary</b> for a certain time in a specific <b>period</b>.
            </nb-card-footer>
        </nb-card>
    </div>
    `
})
export class HeatmapComponent implements OnInit {
  loading = true;
  public map1: L.Map;
  public map2: L.Map;

  selectedPeriod1 = "2017_03_F10";
  selectedPeriod2 = "2020_03_F10";

  constructor(private messinaClient: MessinaClientService) {
  }

  layersControl1 = {
    overlays: {}
  }

  layersControl2 = {
    overlays: {}
  }

  ngOnInit(): void {
    L.AwesomeMarkers.Icon.prototype.options.prefix = 'fa';

    let now = new Date();
    let dayBefore = new Date().setDate(now.getDate() - 1);
    var tzoffset = (new Date()).getTimezoneOffset() * 60000; //offset in milliseconds
    var startDateTime = (new Date(dayBefore - tzoffset)).toISOString().slice(0, -1).split('.').shift() + 'Z';
    var endDateTime = (new Date(now.getTime() - tzoffset)).toISOString().slice(0, -1).split('.').shift() + 'Z';

    //Inizialize Maps
    this.initializeMap1(startDateTime, endDateTime, this.selectedPeriod1);
    this.initializeMap2(startDateTime, endDateTime, this.selectedPeriod2);

    // Loads others layers for Maps
    this.displayStops1();
    this.displayPOIs1();
    this.displayStops2();
    this.displayPOIs2();
  }

  options1 = {
    layers: [
      L.tileLayer("http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png", {
        maxZoom: 18,
        attribution: ""
      })
    ],
    zoom: 11,
    center: L.latLng(38.181529, 15.562588)
  };

  options2 = {
    layers: [
      L.tileLayer("http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png", {
        maxZoom: 18,
        attribution: ""
      })
    ],
    zoom: 11,
    center: L.latLng(38.181529, 15.562588)
  };


  heatOptions = {
    radius: 25,
    max: 1.0,
    blur: 15,
    maxZoom: 18,
    // gradient: { 0.0: 'green', 0.5: 'yellow', 1.0: 'red' },
    minOpacity: 0.7
  }

  layers1 = [];
  layers2 = [];

  onMapReady1(map) {
    var legend = new (L.Control.extend({
      options: { position: 'bottomright' }
    }));

    legend.onAdd = (map) => {
      var div = L.DomUtil.create('div', 'info legend'),
        grades = [0.0, 0.5, 1.0],
        labels = ["Low level", "Medium level", "High level "];

      div.innerHTML = '<div><b>Legend</b></div>';
      for (var i = 0; i < grades.length; i++) {
        div.innerHTML += '<i style="background: ' + this.getLegendColor(grades[i]) + '">'
          + '&nbsp;&nbsp;&nbsp;&nbsp;' + '</i> ' + labels[i] + (i + 1 < grades.length ? '</br>' : '');
      }
      return div;
    };
    legend.addTo(map);
    // this.layers1.push(L.heatLayer( this.heatLayerConverter(geoJsonPoints) , this.heatOptions));
  }

  onMapReady2(map) {
    var legend = new (L.Control.extend({
      options: { position: 'bottomright' }
    }));

    legend.onAdd = (map) => {
      var div = L.DomUtil.create('div', 'info legend'),
        grades = [0.0, 0.5, 1.0],
        labels = ["Low level", "Medium level", "High level "];

      div.innerHTML = '<div><b>Legend</b></div>';
      for (var i = 0; i < grades.length; i++) {
        div.innerHTML += '<i style="background: ' + this.getLegendColor(grades[i]) + '">'
          + '&nbsp;&nbsp;&nbsp;&nbsp;' + '</i> ' + labels[i] + (i + 1 < grades.length ? '</br>' : '');
      }
      return div;
    };
    legend.addTo(map);
    // this.layers2.push(L.heatLayer( this.heatLayerConverter(geoJsonPoints) , this.heatOptions));
  }

  getLegendColor(v: number) {
    let color = 'green';
    if (v == 0.0) {
      color = 'green';
    } else if (v == 0.5) {
      color = 'yellow';
    } else if (v == 1.0) {
      color = 'red';
    }
    return color;
  }


  /**
   * This transform geoJSON into array managed by leaflet for heatmap
   * @param geoJson geoJSON 
   * @returns Points with intensity for leflet heatmap
   */
  private heatLayerConverter(geoJson) {
    var result = [];
    for (let r in geoJson['features']) {
      let coord = geoJson['features'][r].geometry.coordinates;
      result.push([coord[1], coord[0], 0.5]); // lat, lng, intensity 
    }
    // console.log("Points result.length =>" + result.length + "  result: " + result);
    return result;
  }

  updateMap1($event) {
    var startDate = $event['startDate'];
    var endDate = $event['endDate'];
    var selectedPeriod = $event['selectedPeriod'];
    var tzoffset = (new Date()).getTimezoneOffset() * 60000; //offset in milliseconds
    var startLocalISOTime = (new Date(startDate - tzoffset)).toISOString().slice(0, -1).split('.').shift() + 'Z';
    var endLocalISOTime = (new Date(endDate - tzoffset)).toISOString().slice(0, -1).split('.').shift() + 'Z';

    this.initializeMap1(startLocalISOTime, endLocalISOTime, selectedPeriod);
  }

  updateMap2($event) {
    var startDate = $event['startDate'];
    var endDate = $event['endDate'];
    var selectedPeriod = $event['selectedPeriod'];
    var tzoffset = (new Date()).getTimezoneOffset() * 60000; //offset in milliseconds
    var startLocalISOTime = (new Date(startDate - tzoffset)).toISOString().slice(0, -1).split('.').shift() + 'Z';
    var endLocalISOTime = (new Date(endDate - tzoffset)).toISOString().slice(0, -1).split('.').shift() + 'Z';

    this.initializeMap2(startLocalISOTime, endLocalISOTime, selectedPeriod);
  }

  private initializeMap1(startDate, endDate, selectedPeriod) {
    this.loading = true;
    this.messinaClient.getHeatMap(selectedPeriod, startDate, endDate).subscribe(res => {
      this.layers1 = [];
      this.layers1.push(L.heatLayer(this.heatLayerConverter(res), this.heatOptions));
      this.loading = false;
    });
  }

  private initializeMap2(startDate, endDate, selectedPeriod) {
    this.loading = true;
    this.messinaClient.getHeatMap(selectedPeriod, startDate, endDate).subscribe(res => {
      this.layers2 = [];
      this.layers2.push(L.heatLayer(this.heatLayerConverter(res), this.heatOptions));
      this.loading = false;
    });
  }


/**
 * Call API and display a Layer on MAP1 with Stops
 */
  private displayStops1() {
    let clusterData = L.markerClusterGroup();
    this.messinaClient.getStops().subscribe(res => {
      let stopLayer = L.geoJSON(res['results'], {
        pointToLayer: (feature, latlng) => {

          let markerIcon = L.AwesomeMarkers.icon({
            markerColor: 'blue',
            prefix: 'fa',
            extraClasses: 'fas',
            icon: "bus",
            iconColor: "white",
          });

          return L.marker(latlng, { icon: markerIcon }).bindPopup(()=>{
            let str='';
            if(feature.properties.stopName!=undefined && feature.properties.stopName!=null && feature.properties.stopName!=''){
              str+='<p><strong>Stop Name: </strong>'+feature.properties.stopName+'</p>';
            }
            return str
          })
        }
      });
      // this.layers.push(this.stopLayer);
      clusterData.addLayer(stopLayer);
      this.layersControl1.overlays['Layer STOPs'] = clusterData;
    });
  }




  /**
   * Call API and display a Layer on MAP1 with POIs
  */
  private displayPOIs1() {
    let clusterData = L.markerClusterGroup();
    this.messinaClient.getPOI().subscribe(res => {
      let poiLayer = L.geoJSON(res['results'], {
        pointToLayer: (feature, latlng) => {
          let icon = this.getIcon(feature.properties['type']);
          let markerIcon = L.AwesomeMarkers.icon({
            markerColor: 'blue',
            prefix: 'fa',
            extraClasses: 'fas',
            icon: icon,
            iconColor: "white",
          });

          return L.marker(latlng, { icon: markerIcon }).bindPopup(()=>{
            let str='';
            if(feature.properties.category!=undefined && feature.properties.category!=null && feature.properties.category!='') {
              str+='<p><strong>Category: </strong>'+feature.properties.category+'</p>';
            }
            if(feature.properties.name!=undefined && feature.properties.name!=null && feature.properties.name!=''){
              str+='<p><strong>Name: </strong>'+feature.properties.name+'</p>';
            }
            return str
          })
        }
      });
      clusterData.addLayer(poiLayer);
      this.layersControl1.overlays['Layer POIs'] = clusterData;
    });
  }

  /**
 * Call API and display a Layer on MAP2 with Stops
 */
   private displayStops2() {
    let clusterData = L.markerClusterGroup();
    this.messinaClient.getStops().subscribe(res => {
      let stopLayer = L.geoJSON(res['results'], {
        pointToLayer: (feature, latlng) => {

          let markerIcon = L.AwesomeMarkers.icon({
            markerColor: 'blue',
            prefix: 'fa',
            extraClasses: 'fas',
            icon: "bus",
            iconColor: "white",
          });

          return L.marker(latlng, { icon: markerIcon }).bindPopup(()=>{
            let str='';
            if(feature.properties.stopName!=undefined && feature.properties.stopName!=null && feature.properties.stopName!=''){
              str+='<p><strong>Stop Name: </strong>'+feature.properties.stopName+'</p>';
            }
            return str
          })
        }
      });
      // this.layers.push(this.stopLayer);
      clusterData.addLayer(stopLayer);
      this.layersControl2.overlays['Layer STOPs'] = clusterData;
    });
  }


  /**
   * Call API and display a Layer on MAP2 with POIs
  */
  private displayPOIs2() {
    let clusterData = L.markerClusterGroup();
    this.messinaClient.getPOI().subscribe(res => {
      let poiLayer = L.geoJSON(res['results'], {
        pointToLayer: (feature, latlng) => {
          let icon = this.getIcon(feature.properties['type']);
          let markerIcon = L.AwesomeMarkers.icon({
            markerColor: 'blue',
            prefix: 'fa',
            extraClasses: 'fas',
            icon: icon,
            iconColor: "white",
          });

          return L.marker(latlng, { icon: markerIcon }).bindPopup(()=>{
            let str='';
            if(feature.properties.category!=undefined && feature.properties.category!=null && feature.properties.category!='') {
              str+='<p><strong>Category: </strong>'+feature.properties.category+'</p>';
            }
            if(feature.properties.name!=undefined && feature.properties.name!=null && feature.properties.name!=''){
              str+='<p><strong>Name: </strong>'+feature.properties.name+'</p>';
            }
            return str
          })
        }
      });
      clusterData.addLayer(poiLayer);
      this.layersControl2.overlays['Layer POIs'] = clusterData;
    });
  }


  private getIcon(type): string {
    switch (type) {
      case "Retail_Services":
        return "store";
      case "Restaurant":
        return "utensils";
      case "Medical_Facility":
        return "heartbeat";
      case "School":
        return "school";
      case "Sport_Recreation":
        return "running";
      case "City_Town_Center":
        return "university";
      case "Hotel":
        return "h-square";
      case "Enclosed_Carpark":
        return "parking";
      case "Automobile_Facility":
        return "car-side";
      case "Bus_Station":
        return "bus";
      case "Religious_Facility":
        return "pray";
      case "Open_Carpark":
        return "parking";
      case "Forest":
        return "tree";
      case "Cultural_Buildings":
        return "building";
      case "River":
        return "water";
      case "Rail_Station":
        return "train";
      case "Ferry":
        return "ship";
      case "Lake":
        return "water";
      case "Public_Buildings":
        return "building";
      case "WaterElements":
        return "water";
      case "Regio Nat State Park":
        return "tree";
      case "lndustrial_Area":
        return "industry";
      case "Industry":
        return "industry";
      case "Sand_Area":
        return "umbrella-beach";
      case "Tunnel":
        return "archway";
      case "Rail":
        return "train";
    }
    return "map-marker"
  }


}
