import { Component, OnInit } from '@angular/core';
import * as L from 'leaflet';
import 'leaflet-timedimension';
import 'leaflet.awesome-markers';
import {
  NbComponentStatus,
  NbGlobalPhysicalPosition,
  NbGlobalPosition,
  NbToastrService
} from '@nebular/theme';
import { MessinaClientService } from '../service/messina-client.service';

@Component({
  selector: 'ngx-leaflet',
  styleUrls: ['./traffic.component.scss'],
  template: `
    <nb-card [nbSpinner]="loading">
      <nb-card-header>
        Messina Traffic Evolution
        <ngx-traffic-filter (sendDates)="updateMap($event)"></ngx-traffic-filter>
      </nb-card-header>
      <nb-card-body>
      <div class="map"
          leaflet
          [leafletOptions]="options"
          [leafletLayers]="layers"
          [leafletLayersControl]="layersControl"
          [leafletFitBounds]="fitbounds"
          (leafletMapReady)="onMapReady($event)"
          >
        </div>
      </nb-card-body>
      <nb-card-footer>
      This <i>map</i> shows the temporal <b>evolution of traffic flow</b> on some roads entering or leaving the city of <b>Messina</b>.
      </nb-card-footer>
    </nb-card>
  `,
})
export class TrafficComponent implements OnInit {

  constructor(private messinaClient: MessinaClientService, private toastrService: NbToastrService) {
  }

  geojson: any;
  geojsonH: any;
  fitbounds = null;
  instant = null;
  loading = true;

  layers = [];
  layersControl = {
    overlays: {}
  }

  MAX_MINUTES: number;


  ngOnInit(): void {
    L.AwesomeMarkers.Icon.prototype.options.prefix = 'fa';

    this.MAX_MINUTES = 24 * 60 * 1; //TODO - Read from property (1)

    let now = new Date();
    let dayBefore = new Date().setDate(now.getDate() - 1);
    var tzoffset = (new Date()).getTimezoneOffset() * 60000; //offset in milliseconds
    var startDateTime = (new Date(dayBefore - tzoffset)).toISOString().slice(0, -1).split('.').shift() + 'Z';
    var endDateTime = (new Date(now.getTime() - tzoffset)).toISOString().slice(0, -1).split('.').shift() + 'Z';

    this.initializeMap(startDateTime, endDateTime);

    // Loads others layers
    this.displayStops();
    this.displayPOIs();
  }

  private initializeMap(startDate, endDate) {
    this.loading = true;
    this.messinaClient.getTrafficByDateTime(startDate, endDate).subscribe(res => {
      this.trafficMapConverter(res);
      this.loading = false;
    });
  }

  /**
   * Call API and display a Layer on MAP with Stops
   */
   private displayStops() {
    let clusterData = L.markerClusterGroup();
    this.messinaClient.getStops().subscribe(res => {
      let stopLayer = L.geoJSON(res['results'], {
        pointToLayer: (feature, latlng) => {

          let markerIcon = L.AwesomeMarkers.icon({
            markerColor: 'blue',
            prefix: 'fa',
            extraClasses: 'fas',
            icon: "bus",
            iconColor: "white",
          });

          return L.marker(latlng, { icon: markerIcon }).bindPopup(()=>{
            let str='';
            if(feature.properties.stopName!=undefined && feature.properties.stopName!=null && feature.properties.stopName!='')
              str+='<p><strong>Stop Name: </strong>'+feature.properties.stopName+'</p>';
            return str
          })
        }
      });
      // this.layers.push(this.stopLayer);
      clusterData.addLayer(stopLayer);
      this.layersControl.overlays['Layer STOPs'] = clusterData;
    });
  }

  /**
   * Call API and display a Layer on MAP with POIs
   */
   private displayPOIs() {
    let clusterData = L.markerClusterGroup();
    this.messinaClient.getPOI().subscribe(res => {
      let poiLayer = L.geoJSON(res['results'], {
        pointToLayer: (feature, latlng) => {
          let icon = this.getIcon(feature.properties['type']);
          let markerIcon = L.AwesomeMarkers.icon({
            markerColor: 'blue',
            prefix: 'fa',
            extraClasses: 'fas',
            icon: icon,
            iconColor: "white",
          });

          return L.marker(latlng, { icon: markerIcon }).bindPopup(()=>{
            let str='';
            if(feature.properties.category!=undefined && feature.properties.category!=null && feature.properties.category!='') {
              str+='<p><strong>Category: </strong>'+feature.properties.category+'</p>';
            }
            if(feature.properties.name!=undefined && feature.properties.name!=null && feature.properties.name!=''){
              str+='<p><strong>Name: </strong>'+feature.properties.name+'</p>';
            }
            return str
          })
        }
      });
      clusterData.addLayer(poiLayer);
      this.layersControl.overlays['Layer POIs'] = clusterData;
    });
  }

  private getIcon(type): string {
    switch (type) {
      case "Retail_Services":
        return "store";
      case "Restaurant":
        return "utensils";
      case "Medical_Facility":
        return "heartbeat";
      case "School":
        return "school";
      case "Sport_Recreation":
        return "running";
      case "City_Town_Center":
        return "university";
      case "Hotel":
        return "h-square";
      case "Enclosed_Carpark":
        return "parking";
      case "Automobile_Facility":
        return "car-side";
      case "Bus_Station":
        return "bus";
      case "Religious_Facility":
        return "pray";
      case "Open_Carpark":
        return "parking";
      case "Forest":
        return "tree";
      case "Cultural_Buildings":
        return "building";
      case "River":
        return "water";
      case "Rail_Station":
        return "train";
      case "Ferry":
        return "ship";
      case "Lake":
        return "water";
      case "Public_Buildings":
        return "building";
      case "WaterElements":
        return "water";
      case "Regio Nat State Park":
        return "tree";
      case "lndustrial_Area":
        return "industry";
      case "Industry":
        return "industry";
      case "Sand_Area":
        return "umbrella-beach";
      case "Tunnel":
        return "archway";
      case "Rail":
        return "train";
    }
    return "map-marker"
  }

  private trafficMapConverter(messina) {
    this.layers = [];

    this.geojson = {
      "type": "FeatureCollection",
      "features": []
    };

    this.geojsonH = {
      "type": "FeatureCollection",
      "features": [{
        "type": "Feature",
        "properties": {
          "times": []
        },
        "geometry": {
          "type": "LineString",
          "coordinates": []
        }
      }]
    };

    for (let r in messina['roads']) {
      let feature = messina['roads'][r].roadShape;
      feature.properties['roadKey'] = r;
      this.geojson.features.push(feature);
    }

    for (let result of messina['results']) {
      let millis = new Date(result['datetime']).getTime();
      if (this.geojsonH.features[0].properties.times.indexOf(millis) < 0) {
        this.geojsonH.features[0].properties.times.push(millis);
        this.geojsonH.features[0].geometry.coordinates.push([15.529174804687498, 38.20365531807149]);
      }
      for (let d of result['data']) {
        for (let f of this.geojson.features) {
          if (d.roadKey == f.properties['roadKey']) {
            f.properties[millis] = d['JF'];
          }
        }
      }
    }

    if (this.geojson.features.length > 0)
      this.addGeoJSONLayers(this.geojson, this.geojsonH);
  }

  options = {
    layers: [
      L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png',
        {
          maxZoom: 30,
          attribution: '© OpenStreetMap <a href="https://www.openstreetmap.org/copyright" target="_blank">contributors</a>',
          id: 'osm-bright'
        })
    ],
    zoom: 10,
    center: L.latLng(38.20365531807149, 15.529174804687498),
    timeDimension: true,
    timeDimensionControl: true,
    timeDimensionControlOptions: {
      timeSliderDragUpdate: true,
      loopButton: true,
      autoPlay: true,
      playerOptions: {
        transitionTime: 1000,
        loop: true
      }
    }
  };

  addGeoJSONLayers(dataGeojson, hiddenGeojson) {
    let geojsonLayer = L.geoJSON(dataGeojson, {
      style: (feature) => {
        let v = feature.properties[this.instant];
        let color = 'green';

        if (v > 0 && v < 1) {
          color = '#009688';
        } else if (v >= 1 && v < 2) {
          color = '#4caf50';
        } else if (v >= 2 && v < 3) {
          color = '#8bc34a';
        } else if (v >= 3 && v < 4) {
          color = '#cddc39';
        } else if (v >= 4 && v < 5) {
          color = '#ffeb3b';
        } else if (v >= 5 && v < 6) {
          color = '#ffc107';
        } else if (v >= 6 && v < 7) {
          color = '#ff9800';
        } else if (v >= 7 && v < 8) {
          color = '#ff5722';
        } else if (v >= 8 && v < 9) {
          color = '#f44336';
        } else if (v > 9) {
          color = '#970e0e';
        }

        return { color: color };
      }
    })

    this.layers.push(geojsonLayer)
    this.fitbounds = geojsonLayer.getBounds()

    let hiddenLayer = L.timeDimension.layer.geoJson(L.geoJSON(hiddenGeojson, {
      pointToLayer: (feature, latlng) => {
        return L.circleMarker(latlng, {
          radius: 10.0001,
          fillColor: "#ff7800",
          color: "#000",
          weight: 0,
          opacity: 0,
          fillOpacity: 0.0001
        });
      }
    }), {
      updateTimeDimension: true,
      duration: 'PT5M',
      updateTimeDimensionMode: 'replace',
      addlastPoint: true
    }).on('timeload', (event) => {
      this.instant = event.time;
      geojsonLayer.eachLayer(l => geojsonLayer.resetStyle(l));
    })

    this.layers.push(hiddenLayer);
  }

  updateMap($event) {
    var startDate = $event['startDate'];
    var endDate = $event['endDate'];
    // console.log("# $event['startDate'] => " + startDate);
    // console.log("# $event['endDate'] => " + endDate);

    var tzoffset = (new Date()).getTimezoneOffset() * 60000; //offset in milliseconds
    var startLocalISOTime = (new Date(startDate - tzoffset)).toISOString().slice(0, -1).split('.').shift() + 'Z';
    var endLocalISOTime = (new Date(endDate - tzoffset)).toISOString().slice(0, -1).split('.').shift() + 'Z';
    // console.log('# LocalISOTime - Start Date => ' + startLocalISOTime);
    // console.log('# LocalISOTime - End Date => ' + endLocalISOTime);

    var datesIntervalDiffMinutes = this.calculateDiff(startDate, endDate);
    let ENABLE_DATE_RANGE = false; //TODO - Read from property (2)
    if (ENABLE_DATE_RANGE && datesIntervalDiffMinutes > this.MAX_MINUTES) {
      // console.log("# Skip Inizialize Ma p- Dates Interval should be lower/equals to "+ this.MAX_MINUTES + " minutes. DatesIntervalDiff(minutes): " + datesIntervalDiffMinutes );
      this.makeToast(this.MAX_MINUTES, datesIntervalDiffMinutes);
    } else {
      // console.log("# Inizialize Map");
      this.initializeMap(startLocalISOTime, endLocalISOTime);
    }

  }


  /**
   * This function build the Legend on event onMapReady
   * @param map Map Leaflet
   */
  onMapReady(map: L.Map) {
    var legend = new (L.Control.extend({
      options: { position: 'bottomright' }
    }));

    legend.onAdd = (map) => {
      var div = L.DomUtil.create('div', 'info legend'),
        grades = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
        labels = ["Jam Factor"];

      div.innerHTML = '<div><b>Legend</b></div>';
      for (var i = 0; i < grades.length; i++) {
        div.innerHTML += '<i style="background: ' + this.getLegendColor(grades[i]) + '">' + '&nbsp;&nbsp;&nbsp;&nbsp;' + '</i> ' + labels[0] + ': [' + grades[i] + ' – ' + (grades[i] + 1) + (i + 1 < grades.length ? '[</br>' : ']');
      }
      return div;
    };
    legend.addTo(map);
  }

  getLegendColor(v: number) {
    let color = 'green';
    if (v == 1) {
      color = '#009688';
    } else if (v == 2) {
      color = '#4caf50';
    } else if (v == 3) {
      color = '#8bc34a';
    } else if (v == 4) {
      color = '#cddc39';
    } else if (v == 5) {
      color = '#ffeb3b';
    } else if (v == 6) {
      color = '#ffc107';
    } else if (v == 7) {
      color = '#ff9800';
    } else if (v == 8) {
      color = '#ff5722';
    } else if (v == 9) {
      color = '#970e0e';
    }
    return color;
  }


  /**
   * @param startDate Start Date
   * @param endDate End date 
   * @returns Return difference in minutes between end and start dates
   */
  calculateDiff(startDate, endDate) {
    // let days = Math.floor((endDate.getTime() - startDate.getTime()) / 1000 / 60 / 60 / 24);
    let minutes = Math.floor((endDate.getTime() - startDate.getTime()) / 1000 / 60);
    return minutes;
  }


  makeToast(maxMinutes: number, cminutes: number) {
    this.showToast(this.status, "Check filters", "Reduce the Range (EndDate - StartDate), currently " +
      "range is " + (cminutes / 60).toFixed(2) + " hours grather then max " + maxMinutes / 60 + " hours !");
  }

  destroyByClick = true;
  duration = 5000;
  hasIcon = true;
  position: NbGlobalPosition = NbGlobalPhysicalPosition.TOP_RIGHT;
  preventDuplicates = false;
  status: NbComponentStatus = 'info';

  private showToast(type: NbComponentStatus, title: string, body: string) {
    const config = {
      status: type,
      destroyByClick: this.destroyByClick,
      duration: this.duration,
      hasIcon: this.hasIcon,
      position: this.position,
      preventDuplicates: this.preventDuplicates,
    };
    const titleContent = title ? ` ${title}` : '';

    this.toastrService.show(body, `${titleContent}`, config);
  }
}

