import { Component } from "@angular/core";
import * as L from "leaflet";
import "leaflet-draw";
import { NetworkService } from "../utils/services/network.service";
import { NbToastrService } from "@nebular/theme";

@Component({
  selector: "ngx-edit-network-map",
  templateUrl: "./edit-network-map.component.html",
  styleUrls: ["./edit-network-map.component.scss"],
})
export class EditNetworkMapComponent {
  public map: L.Map;
  public allLinestrings: Object[] = [];
  public allLayers: L.Layer[] = [];
  geoJsonStyleWeight: number = 4;
  geoJsonStyleOpacity: number = 0.5;
  btnAddSelectedBikeLanesDisabled: boolean = true;
  loading: boolean = false;
  options = {
    layers: [
      L.tileLayer("http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png", {
        maxZoom: 19,
        attribution: "...",
      }),
    ],
    zoom: 15,
    center: L.latLng({ lat: 43.263, lng: -2.935 }),
  };

  constructor(
    private toastrService: NbToastrService,
    private networkService: NetworkService
  ) {}

  onMapReady(map: L.Map) {
    this.map = map;
    const drawControl = new L.Control.Draw({
      draw: {
        polygon: false,
        marker: false,
        circlemarker: false,
        rectangle: false,
        circle: false,
      },
    });

    this.map.addControl(drawControl);
    this.map.on(L.Draw.Event.CREATED, (e: any) => {
      const type = e.layerType;
      if (type === "polyline") {
        this.allLayers.push(e.layer);
        (e.layer as L.Layer).addEventListener("click", (e) => {
          e.target["highlighted"]
            ? this.resetFeature(e.target)
            : this.highlightFeature(e.target);
        });
        this.map.addLayer(this.allLayers[this.allLayers.length - 1]);
        this.allLinestrings.push(e.layer.editing.latlngs[0]);
        this.showInfoNotify();
      }
    });
  }

  onAddSelectedBikeLanes() {
    this.btnAddSelectedBikeLanesDisabled = true;
    this.loading = true;
    let tmpFeatures = [];

    this.allLayers.forEach((layer) => {
      if (layer["highlighted"] == true) {
        let coordinatesWithOneBracketLess = [];
        layer["_latlngs"].forEach((point) => {
          coordinatesWithOneBracketLess.push([point.lng, point.lat]);
        });
        tmpFeatures.push({
          type: "Feature",
          properties: {},
          geometry: {
            type: "Polygon",
            coordinates: coordinatesWithOneBracketLess,
          },
        });
      }
    });

    const tmpGeojson: JSON = <JSON>(<unknown>{
      type: "FeatureCollection",
      features: tmpFeatures,
    });

    this.toastrService.show(`Sending to server`, `Info`, {
      status: "primary",
    });
    this.networkService.postGenerateBikeLanes(tmpGeojson, true, 1.5).subscribe(
      (res) => {
        this.toastrService.show(`Server replies success`, `Success`, {
          status: "success",
        });
        this.btnAddSelectedBikeLanesDisabled = false;
        this.loading = false;
      },
      (error) => {
        console.log("🚀☢ ~ error", error);
        this.toastrService.show(
          `${error["message"]}`,
          `${error["statusText"]}`,
          { status: "danger" }
        );
        this.btnAddSelectedBikeLanesDisabled = false;
        this.loading = false;
      }
    );
  }

  onReset() {
    this.allLinestrings = [];
    this.allLayers.forEach((element) => {
      this.map.removeLayer(element);
    });
    this.allLayers = [];
    this.btnAddSelectedBikeLanesDisabled = true;
    this.toastrService.show("Cleared", "Reset", {
      status: "danger",
    });
  }

  private highlightFeature(target) {
    target["highlighted"] = true;
    target.setStyle({
      weight: 10,
      opacity: 1.0,
    });
    this.btnAddSelectedBikeLanesDisabled = false;
    this.showInfoNotify();
  }

  private resetFeature(target) {
    target["highlighted"] = false;
    target.setStyle({
      weight: this.geoJsonStyleWeight,
      opacity: this.geoJsonStyleOpacity,
    });
    this.evaluateAtLeastOneHighlighted();
  }

  public evaluateAtLeastOneHighlighted() {
    let atLeastOneHighlighted: boolean = false;

    this.allLayers.forEach((layer, index) => {
      if (layer["highlighted"] == true) {
        atLeastOneHighlighted = true;
      }
    });

    if (atLeastOneHighlighted) {
      this.btnAddSelectedBikeLanesDisabled = false;
    } else {
      this.btnAddSelectedBikeLanesDisabled = true;
    }
  }

  public showInfoNotify() {
    let n = 0;
    this.allLayers.forEach((layer, index) => {
      if (layer["highlighted"] == true) {
        n++;
      }
    });
    this.toastrService.show(
      `Currently drawn ${this.allLinestrings.length} polylines, ${n} of which selected.`,
      "Polyline Added",
      {
        status: "info",
      }
    );
  }
}
