import {AnimationEvent} from '@angular/animations';
import {Component, OnInit, TemplateRef, ViewChild, ViewRef} from '@angular/core';
import {AbstractControl, FormBuilder, Validators} from '@angular/forms';
import {NbAccordionItemComponent} from '@nebular/theme';
import {RxStompService} from '@stomp/ng2-stompjs';
import * as L from 'leaflet';
import {BoundsTuple, LeafletAreaSelectedEvent} from '../typings';
import {socketStatusAnimation} from '../utils/animations/socket-status.animation';
import {City} from '../utils/data/city';
import {Network} from '../utils/data/network';
import {ViewContainerDirective} from '../utils/directives/view-container.directive';
import {CityService} from '../utils/services/city.service';
import {NetworkService} from '../utils/services/network.service';

@Component({
  selector: 'ngx-network-creation',
  templateUrl: './network-creation.component.html',
  styleUrls: ['./network-creation.component.scss'],
  animations: [socketStatusAnimation],
})
export class NetworkCreationComponent implements OnInit {
  networks: Network[];
  formMap: L.Map;
  viewMap: L.Map;
  formMapRectangle: L.Rectangle;
  viewMapRectangle: L.Rectangle;
  city: City;

  @ViewChild("createNetworkAccordionItem", {static: true}) createNetworkAccordionItem: NbAccordionItemComponent;
  @ViewChild("viewMapTemplate", {static: true}) viewMapTemplate: TemplateRef<any>;
  viewMapView: ViewRef;

  networkForm = this.fb.group({
    name: ['', Validators.required],
    description: [''],
    bounds: [null, Validators.required]
  })

  mapOptionsForm = {
    layers: [
      L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {maxZoom: 18, attribution: '...'}),
    ],
    zoom: 14,
    center: new L.LatLng(0, 0),
  };

  mapOptionsView = {
    layers: [
      L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {maxZoom: 18, attribution: '...'}),
    ],
    zoom: 14,
    center: new L.LatLng(52.5086, 13.3704),
  };

  constructor(private networkService: NetworkService,
              private cityService: CityService,
              private fb: FormBuilder,
              private rxStompService: RxStompService) {
  }

  ngOnInit(): void {
    this.viewMapView = this.viewMapTemplate.createEmbeddedView(null);

    this.cityService.getSelectedCity().subscribe(city => {
      this.city = city;
      this.formMap?.setView(new L.LatLng(this.city.coordinates.lat, this.city.coordinates.lon), 15);
      this.cityService.getCityNetworks(city.cityId).subscribe(networks => this.networks = networks);
      console.log(this.networks)
    })

    this.rxStompService.watch("/topic/networkProcessingStatus").subscribe(updatedNetworkFrame => {
      let updatedNetwork = JSON.parse(updatedNetworkFrame.body);
      let index = this.networks.findIndex(network => network.id == updatedNetwork.id);
      if (index == -1) {
        this.networks.push(Network.deserialize(updatedNetwork));
      } else {
        this.networks[index] = Network.deserialize(updatedNetwork);
        this.networks[index].conversionWSMessage = updatedNetworkFrame.headers.conversionWSMessage;
        this.networks[index].animationState = this.networks[index].animationState == "hidden" ? "show" : "replaceHide";
      }
    });
  }

  onMapReadyForm(map: L.Map) {
    this.formMap = map;
    this.formMap?.setView(new L.LatLng(this.city.coordinates.lat, this.city.coordinates.lon), 14);
    (map as any).selectArea.enable()
    map.on("areaselected", (bounds: LeafletAreaSelectedEvent) => {
      console.log()
      this.bounds.setValue(bounds.bounds.toBBoxString().split(",").map(value => +value) as BoundsTuple)
      this.formMapRectangle?.remove()
      this.formMapRectangle = L.rectangle(bounds.bounds, {
        fill: true,
        fillColor: "#ffffff",
        fillOpacity: 0.5,
        color: "#38f",
        weight: 2
      }).addTo(this.formMap);
    })
  }

  onMapReadyView(map: L.Map) {
    console.log("View map ready");
    this.viewMap = map;
  }

  /*changeMapCentre(value: string) {
    this.formMap?.setView(this.mapCords[value], 14);
    this.bounds.reset();
    this.formMapRectangle?.remove();
  }*/

  onSubmit() {
    let network = Network.deserialize(this.networkForm.value);
    network.cityId = this.city.cityId;
    this.networkForm.reset();
    this.formMapRectangle?.remove();
    this.createNetworkAccordionItem.close();
    this.networkService.createNetwork(network).subscribe(value => {
      this.networks.push(value)
    }, err => console.log(err)); //  TODO add proper error handling
  }

  runClick() {
    // this.simulationService.runSimulation(simulation).subscribe(simulation => {
    //   let index = this.simulations.findIndex(simulationOld => simulationOld.id == simulation.id);
    //   this.simulations[index] = simulation;
    // })
  }

  viewAccordionOpen(open: boolean, viewMapContainer: ViewContainerDirective, i: number) {
    if (!open) {
      viewMapContainer.container.insert(this.viewMapView);
      this.viewMap.setView(new L.LatLng(this.city.coordinates.lat, this.city.coordinates.lon), 14);
      this.viewMapRectangle?.remove();
      this.viewMapRectangle = L.rectangle(this.networks[i].leafletBounds).addTo(this.viewMap);
    }
  }

  //  HACK Makes sure leaflet gets initialized. There must be better way to do this.
  initMap(viewMapContainer: ViewContainerDirective) {
    if (!this.viewMap) {
      viewMapContainer.container.insert(this.viewMapView);
    }
  }

  onAnimationEventDone(event: AnimationEvent, index: number) {
    if (event.toState == "show" && this.networks[index].animationState == "show")
      this.networks[index].animationState = "shown";
    else if (event.toState == "shown" && this.networks[index].animationState == "shown")
      this.networks[index].animationState = "hide";
    else if (event.toState == "hide" && this.networks[index].animationState == "hide")
      this.networks[index].animationState = "hidden";
    else if (event.toState == "replaceHide" && this.networks[index].animationState == "replaceHide")
      this.networks[index].animationState = "replaceShow";
    else if (event.toState == "replaceShow" && this.networks[index].animationState == "replaceShow")
      this.networks[index].animationState = "shown";
  }

  onAnimationEventStart(event: AnimationEvent, index: number) {
    if (event.toState == "show" || event.toState == "replaceShow") {
      (event.element as HTMLElement).innerHTML = this.networks[index].conversionWSMessage
    }
  }

  //  Boilerplate code for easier form elements access
  get name() {
    return this.networkForm.get("name")
  }

  get description() {
    return this.networkForm.get("description")
  }

  get bounds() {
    return this.networkForm.get("bounds")
  }

  checkValid(control: AbstractControl) {
    return control.invalid && (control.dirty || control.touched)
  }

}
