import {
  Component,
  Input,
  TemplateRef,
  OnChanges,
  OnInit,
} from "@angular/core";
import * as L from "leaflet";
import { SimulationService } from "../utils/services/simulation.service";
import * as MapColors from "./map-colors-local";
import { legendInverted, ratiosBetweenGrades } from "./results-map-settings";
import { NbToastrService, NbDialogService } from "@nebular/theme";
import { Simulation } from "../utils/data/simulation";
import { NetworkService } from "../utils/services/network.service";
import { FormControl, FormGroup } from "@angular/forms";

@Component({
  selector: "ngx-results-map",
  templateUrl: "./results-map.component.html",
  styleUrls: ["./results-map.component.scss"],
})
export class ResultsMapComponent implements OnInit, OnChanges {
  @Input() selectedSimulation: Simulation;
  public map: L.Map;
  options = {
    layers: [
      L.tileLayer("http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png", {
        maxZoom: 18,
        attribution: "...",
      }),
    ],
    zoom: 5,
    center: L.latLng({ lat: 50, lng: 14 }),
  };
  mapLegend = new L.Control({ position: "bottomright" });
  checkedRadioButton: string = "";
  extractingKeysLoopLimit: number = 3498;
  geojsonLoading: boolean = false;
  geoJsonStyleWeight: number = 5;
  geoJsonStyleOpacity: number = 0.8;
  grayscale: number = 50;
  loadGeojsons: Object[] = [];
  maxKeyValues: Object = {};
  brinkValues: Object = {};
  newBrinkValuesForm = new FormGroup({
    legend_0: new FormControl(undefined),
    legend_1: new FormControl(undefined),
    legend_2: new FormControl(undefined),
    legend_3: new FormControl(undefined),
    legend_4: new FormControl(undefined),
    legend_5: new FormControl(undefined),
  });
  newBrinkValuesApplied: boolean = false;
  radioButtonLayersControl: any;
  selectedGeojson: string = "";

  constructor(
    private simulationService: SimulationService,
    private toastrService: NbToastrService,
    private dialogService: NbDialogService,
    private networkService: NetworkService
  ) {}
  ngOnInit(): void {
    this.newBrinkValuesForm.disable();
  }

  onMapReady(map: L.Map) {
    this.map = map;
    this.grayscaleChange();

    setTimeout(() => {
      this.map.invalidateSize();
      this.networkService.getMapInfo().subscribe((res) => {
        this.setMapPositionAndZoom(res);
      });
    }, 50);
    // setTimeout(() => {
    //   console.log(
    //     "%cTemporary",
    //     "background: peru; font-weight: bold; color: black; font-family: serif; padding: 0 10px;"
    //   );
    //   const celkomplet: HTMLElement = document.querySelectorAll(
    //     ".buttons"
    //   )[0] as HTMLElement;
    //   celkomplet.style.display = "none";
    // }, 825);
    // setTimeout(() => {
    //   const celkomplet1: HTMLElement = document.querySelectorAll(
    //     ".appearance-filled.size-medium.shape-rectangle.status-primary.ng-star-inserted.nb-transition"
    //   )[1] as HTMLElement;
    //   celkomplet1.click();
    // }, 1000);
    // setTimeout(() => {
    //   const celkomplet2: HTMLElement = document.querySelectorAll(
    //     "#loadGeojsonButtonsContainer button"
    //   )[1] as HTMLElement;
    //   celkomplet2.click();
    // }, 1050);
    // setTimeout(() => {
    //   const celkomplet3: HTMLElement = document.querySelectorAll(
    //     ".leaflet-control-layers-selector"
    //   )[8] as HTMLElement;
    //   celkomplet3.click();
    // }, 2228);
  }

  onLoadClick(selectedKpi: string) {
    this.selectedGeojson = selectedKpi;
    this.resetLayersAndButton();
    this.setLoadGeojsons();

    if (this.radioButtonLayersControl)
      this.radioButtonLayersControl.remove(this.map);

    this.loadGeojsons.forEach((geojson, index) => {
      if (geojson["buttonValue"] == selectedKpi) {
        if (this.loadGeojsons[index]["response"] == undefined) {
          this.toastrService.show("This might take a while", "Info", {
            status: "info",
          });
          this.geojsonLoading = true;
          this.simulationService
            .getGeojson(this.selectedSimulation["id"].toString(), selectedKpi)
            .subscribe((res) => {
              this.loadGeojsons[index]["response"] = res;
              this.handleResponse(this.loadGeojsons[index]["response"]);
            }),
            (error) => {
              this.toastrService.show(
                error["message"],
                "Error Getting Geojson",
                {
                  status: "danger",
                }
              );
              this.geojsonLoading = false;
            };
        } else {
          this.handleResponse(this.loadGeojsons[index]["response"]);
        }
      }
    });
  }

  handleResponse(res) {
    if (res["message"] == "Calculating...") {
      this.toastrService.show(
        "Server says: Calculating the KPI visualization. GEOJSON loading will not be completed.",
        "Geojson not ready",
        {
          status: "danger",
          duration: 10000,
        }
      );
      this.geojsonLoading = false;
      return;
    } else if (res["message"] == "Calculating the KPI visualization.") {
      this.toastrService.show(
        "Server says: Calculating the KPI visualization. GEOJSON loading will not be completed.",
        "Geojson not ready",
        {
          status: "danger",
          duration: 10000,
        }
      );
      this.geojsonLoading = false;
      return;
    } else if (res["message"] == "Please prepare visualizations first.") {
      this.toastrService.show(
        "The visualization is not prepared yet, try again in a few minutes.",
        "Geojson not ready",
        {
          status: "danger",
          duration: 10000,
        }
      );
      this.geojsonLoading = false;
      return;
    } else if (res["message"] && res["message"].length > 0) {
      this.toastrService.show(
        "Geojson might not be ready",
        "API response contains a message",
        {
          status: "warning",
        }
      );
    }

    this.toastrService.show("", "Geojson Loaded", {
      status: "success",
    });

    const keys = this.extractKeys(res);
    const propertyGeojsons = this.populatePropertyGeojsons(res, keys);
    let obj = {};

    for (let i = 0; i < propertyGeojsons.length; i++) {
      if (
        propertyGeojsons[i].myKey == "link_id" ||
        propertyGeojsons[i].myKey == "modes" ||
        propertyGeojsons[i].myKey == "internal_travel_by_mode"
      )
        continue;
      obj[`${keys[i]}`] = propertyGeojsons[i];
    }

    this.radioButtonLayersControl = new L.Control.Layers(obj, null, {
      collapsed: false,
    });
    this.map.addControl(this.radioButtonLayersControl);
    this.addListenerToRadioButtonLayersControlAndSetLegend();
    this.geojsonLoading = false;
  }

  onApplyNewValuesSubmit() {
    // here we are forcing the user to make his choices again
    if (this.radioButtonLayersControl)
      this.radioButtonLayersControl.remove(this.map);
    this.resetLayersAndButton();

    this.selectedGeojson = undefined;

    // when the user will newly click on things, new geojsons will be created
    this.newBrinkValuesApplied = true;
  }

  private highlightFeature(e, feature) {
    const layer = e.target;
    let text = "";

    if (feature["cityWide"]) {
      for (let p in feature["cityWide"]["pollution"]) {
        const tmp = this.limitMyDigits(feature["cityWide"]["pollution"][p]);
        if (p == this.checkedRadioButton) {
          text += `<u><strong>${p}:</strong> ${tmp}</u><br>`;
        } else {
          text += `<strong>${p}:</strong> ${tmp}<br>`;
        }
        text += `<span style='display: inline-block; margin-bottom: 7px; color: #666'>(unrounded) ${feature["cityWide"]["pollution"][p]}</span><br>`;
      }
      for (let p in feature["cityWide"]["traffic"]) {
        const tmp = this.limitMyDigits(feature["cityWide"]["traffic"][p]);
        if (p == this.checkedRadioButton) {
          text += `<u><strong>${p}:</strong> ${tmp}</u><br>`;
        } else {
          text += `<strong>${p}:</strong> ${tmp}<br>`;
        }
        text += `<span style='display: inline-block; margin-bottom: 7px; color: #666'>(unrounded) ${feature["cityWide"]["traffic"][p]}</span><br>`;
      }
    }

    Object.entries(feature).forEach((element) => {
      if (element[0] == "capacity") {
        // exception for capacity checking for value 9999
        if (feature["capacity"] == "9999.0" || feature["capacity"] == "9999") {
          text += `<span style='display: inline-block; margin-bottom: 7px; font-weight: bold;'>Oops! Looks like you clicked on an transparent bus line with capacity of 9999!</span><br>`;
        }
        const tmp = this.limitMyDigits(feature["capacity"]);
        if ("capacity" == this.checkedRadioButton) {
          text += `<u><strong>capacity:</strong> ${tmp}</u><br>`;
        } else {
          text += `<strong>capacity:</strong> ${tmp}<br>`;
        }
        text += `<span style='display: inline-block; margin-bottom: 7px; color: #666'>(unrounded) ${feature["capacity"]}</span><br>`;
      } else {
        const tmp = this.limitMyDigits(element[1] as number).toString();
        if (element[0] == this.checkedRadioButton) {
          text += `<u><strong>${element[0]}:</strong> ${tmp}</u><br>`;
        } else {
          text += `<strong>${element[0]}:</strong> ${tmp}<br>`;
        }
        text += `<span style='display: inline-block; margin-bottom: 7px; color: #666'>(unrounded) ${element[1]}</span><br>`;
      }
    });

    if (!text.length)
      text = "<span style='color: lightslategray';>No data available.</span>";
    layer.bindPopup(text);
    layer.setStyle({
      weight: 10,
      opacity: 1.0,
    });
  }

  private resetFeature(e) {
    const layer = e.target;

    layer.setStyle({
      weight: this.geoJsonStyleWeight,
      opacity: this.geoJsonStyleOpacity,
    });
  }

  public extractKeys(jsonFile) {
    let allKeys = [];
    for (
      let f = 0;
      f < jsonFile["features"].length && f < this.extractingKeysLoopLimit;
      f++
    ) {
      let iterator;
      Object.keys(jsonFile["features"][f]["properties"]).forEach((element) => {
        if (!allKeys.includes(element)) {
          allKeys.push(element);
        }
      });
      if (jsonFile["features"][f]["properties"]["capacity"]) {
        if (!allKeys.includes("capacity")) {
          allKeys.push("capacity");
        }
      }
      if (
        jsonFile["features"][f]["properties"]["cityWide"] &&
        jsonFile["features"][f]["properties"]["cityWide"]["pollution"]
      ) {
        iterator = Object.keys(
          jsonFile["features"][f]["properties"]["cityWide"]["pollution"]
        );
        for (let g = 0; g < iterator.length; g++) {
          if (!allKeys.includes(iterator[g])) {
            allKeys.push(iterator[g]);
          }
        }
      }
      if (
        jsonFile["features"][f]["properties"]["cityWide"] &&
        jsonFile["features"][f]["properties"]["cityWide"]["traffic"]
      ) {
        iterator = Object.keys(
          jsonFile["features"][f]["properties"]["cityWide"]["traffic"]
        );
        for (let g = 0; g < iterator.length; g++) {
          if (!allKeys.includes(iterator[g])) {
            allKeys.push(iterator[g]);
          }
        }
      }
    }
    jsonFile.allKeys = allKeys;
    return allKeys;
  }

  public populatePropertyGeojsons(jsonFile, keys) {
    let res = [];
    this.populateMaxKeyValues(jsonFile, keys);

    for (let f = 0; f < keys.length; f++) {
      let tmp: Object = {};

      if (
        this.maxKeyValues["maxValue_" + keys[f]] == undefined ||
        this.maxKeyValues["maxValue_" + keys[f]] == 0
      ) {
        tmp = L.geoJSON(<any>{
          type: "FeatureCollection",
          features: [],
        });
      } else {
        if (this.newBrinkValuesApplied) {
          let l = [];
          l.push(this.newBrinkValuesForm.get("legend_0").value);
          l.push(this.newBrinkValuesForm.get("legend_1").value);
          l.push(this.newBrinkValuesForm.get("legend_2").value);
          l.push(this.newBrinkValuesForm.get("legend_3").value);
          l.push(this.newBrinkValuesForm.get("legend_4").value);
          this.brinkValues[keys[f]] = l;

          if (keys[f] == "capacity") {
            tmp = L.geoJSON(<any>jsonFile, {
              style: (feature) => ({
                weight: this.geoJsonStyleWeight,
                color: MapColors.getColorHighToLow(
                  feature["properties"]["capacity"],
                  keys[f],
                  l,
                  parseInt(feature["properties"]["capacity"]) == 9999
                    ? true
                    : false
                ),
                opacity: this.geoJsonStyleOpacity,
              }),
              onEachFeature: (feature, layer) =>
                layer.on({
                  mouseover: (e) =>
                    this.highlightFeature(e, feature.properties),
                  mouseout: (e) => this.resetFeature(e),
                }),
            });
          } else if (
            keys[f] == "dailyInternalBikeTravels" ||
            keys[f] == "pedestrianTravelTime"
          ) {
            tmp = L.geoJSON(<any>jsonFile, {
              style: (feature) => ({
                weight: this.geoJsonStyleWeight,
                color: MapColors.getColor(
                  feature["properties"]["cityWide"]["traffic"][keys[f]] ||
                    feature["properties"][keys[f]],
                  keys[f],
                  l,
                  false
                ),
                opacity: this.geoJsonStyleOpacity,
              }),
              onEachFeature: (feature, layer) =>
                layer.on({
                  mouseover: (e) =>
                    this.highlightFeature(e, feature.properties),
                  mouseout: (e) => this.resetFeature(e),
                }),
            });
          } else if (legendInverted.indexOf(keys[f]) > -1) {
            tmp = L.geoJSON(<any>jsonFile, {
              style: (feature) => ({
                weight: this.geoJsonStyleWeight,
                color: MapColors.getColorHighToLow(
                  feature["properties"],
                  keys[f],
                  l,
                  false
                ),
                opacity: this.geoJsonStyleOpacity,
              }),
              onEachFeature: (feature, layer) =>
                layer.on({
                  mouseover: (e) =>
                    this.highlightFeature(e, feature.properties),
                  mouseout: (e) => this.resetFeature(e),
                }),
            });
          } else {
            tmp = L.geoJSON(<any>jsonFile, {
              style: (feature) => ({
                weight: this.geoJsonStyleWeight,
                color: MapColors.getColor(
                  feature["properties"],
                  keys[f],
                  l,
                  false
                ),
                opacity: this.geoJsonStyleOpacity,
              }),
              onEachFeature: (feature, layer) =>
                layer.on({
                  mouseover: (e) =>
                    this.highlightFeature(e, feature.properties),
                  mouseout: (e) => this.resetFeature(e),
                }),
            });
          }
        } else {
          // default, newBrinkValuesForm not applied
          let l = [];
          ratiosBetweenGrades.forEach((r) => {
            l.push(
              this.roundMyNumbers(this.maxKeyValues[`maxValue_${keys[f]}`] / r)
            );
          });
          this.brinkValues[keys[f]] = l;

          if (keys[f] == "capacity") {
            tmp = L.geoJSON(<any>jsonFile, {
              style: (feature) => ({
                weight: this.geoJsonStyleWeight,
                color: MapColors.getColorHighToLow(
                  feature["properties"]["capacity"],
                  keys[f],
                  l,
                  parseInt(feature["properties"]["capacity"]) == 9999
                    ? true
                    : false
                ),
                opacity: this.geoJsonStyleOpacity,
              }),
              onEachFeature: (feature, layer) =>
                layer.on({
                  mouseover: (e) =>
                    this.highlightFeature(e, feature.properties),
                  mouseout: (e) => this.resetFeature(e),
                }),
            });
          } else if (
            keys[f] == "dailyInternalBikeTravels" ||
            keys[f] == "pedestrianTravelTime"
          ) {
            tmp = L.geoJSON(<any>jsonFile, {
              style: (feature) => ({
                weight: this.geoJsonStyleWeight,
                color: MapColors.getColor(
                  feature["properties"]["cityWide"]["traffic"][keys[f]] ||
                    feature["properties"][keys[f]],
                  keys[f],
                  l,
                  false
                ),
                opacity: this.geoJsonStyleOpacity,
              }),
              onEachFeature: (feature, layer) =>
                layer.on({
                  mouseover: (e) =>
                    this.highlightFeature(e, feature.properties),
                  mouseout: (e) => this.resetFeature(e),
                }),
            });
          } else if (legendInverted.indexOf(keys[f]) > -1) {
            tmp = L.geoJSON(<any>jsonFile, {
              style: (feature) => ({
                weight: this.geoJsonStyleWeight,
                color: MapColors.getColorHighToLow(
                  feature["properties"],
                  keys[f],
                  l,
                  false
                ),
                opacity: this.geoJsonStyleOpacity,
              }),
              onEachFeature: (feature, layer) =>
                layer.on({
                  mouseover: (e) =>
                    this.highlightFeature(e, feature.properties),
                  mouseout: (e) => this.resetFeature(e),
                }),
            });
          } else {
            tmp = L.geoJSON(<any>jsonFile, {
              style: (feature) => ({
                weight: this.geoJsonStyleWeight,
                color: MapColors.getColor(
                  feature["properties"],
                  keys[f],
                  l,
                  false
                ),
                opacity: this.geoJsonStyleOpacity,
              }),
              onEachFeature: (feature, layer) =>
                layer.on({
                  mouseover: (e) =>
                    this.highlightFeature(e, feature.properties),
                  mouseout: (e) => this.resetFeature(e),
                }),
            });
          }
        }
      }
      tmp["myKey"] = keys[f];
      res.push(tmp);
    }
    return res;
  }

  public addListenerToRadioButtonLayersControlAndSetLegend() {
    this.map.on("baselayerchange", (e) => {
      this.setLegend(e["name"]);
      this.checkedRadioButton = e["name"];
      this.newBrinkValuesForm.enable();
    });
  }

  public populateMaxKeyValues(jsonFile: Object, keys: Object[]) {
    console.log(
      " if two or more geojsons have 'capacity', then the it will not work correctly"
    );
    for (let g = 0; g < jsonFile["features"].length; g++) {
      for (let f = 0; f < keys.length; f++) {
        if (
          jsonFile["features"][g]["properties"]["cityWide"] &&
          jsonFile["features"][g]["properties"]["cityWide"]["pollution"]
        ) {
          if (this.maxKeyValues[`maxValue_${keys[f]}`] == undefined) {
            this.maxKeyValues[`maxValue_${keys[f]}`] =
              jsonFile["features"][g]["properties"]["cityWide"]["pollution"][
                keys[f]
              ];
          }
          if (
            jsonFile["features"][g]["properties"]["cityWide"]["pollution"][
              keys[f]
            ] > this.maxKeyValues[`maxValue_${keys[f]}`]
          ) {
            this.maxKeyValues[`maxValue_${keys[f]}`] =
              jsonFile["features"][g]["properties"]["cityWide"]["pollution"][
                keys[f]
              ];
          }
        }
        if (
          jsonFile["features"][g]["properties"]["cityWide"] &&
          jsonFile["features"][g]["properties"]["cityWide"]["traffic"]
        ) {
          if (this.maxKeyValues[`maxValue_${keys[f]}`] == undefined) {
            this.maxKeyValues[`maxValue_${keys[f]}`] =
              jsonFile["features"][g]["properties"]["cityWide"]["traffic"][
                keys[f]
              ];
          }
          if (
            jsonFile["features"][g]["properties"]["cityWide"]["traffic"][
              keys[f]
            ] > this.maxKeyValues[`maxValue_${keys[f]}`]
          ) {
            this.maxKeyValues[`maxValue_${keys[f]}`] =
              jsonFile["features"][g]["properties"]["cityWide"]["traffic"][
                keys[f]
              ];
          }
        }
        if (jsonFile["features"][g]["properties"]["capacity"]) {
          const parsed = parseInt(
            jsonFile["features"][g]["properties"]["capacity"]
          );
          if (parsed != 9999) {
            if (this.maxKeyValues[`maxValue_capacity`] == undefined) {
              this.maxKeyValues[`maxValue_capacity`] = parsed;
            }
            if (parsed > this.maxKeyValues[`maxValue_capacity`]) {
              this.maxKeyValues[`maxValue_capacity`] = parsed;
            }
          }
        }
        if (jsonFile["features"][g]["properties"][keys[f]]) {
          if (this.maxKeyValues[`maxValue_${keys[f]}`] == undefined) {
            this.maxKeyValues[`maxValue_${keys[f]}`] =
              jsonFile["features"][g]["properties"][keys[f]];
          }
          if (
            jsonFile["features"][g]["properties"][keys[f]] >
            this.maxKeyValues[`maxValue_${keys[f]}`]
          ) {
            this.maxKeyValues[`maxValue_${keys[f]}`] =
              jsonFile["features"][g]["properties"][keys[f]];
          }
        }
      }
    }
  }

  public limitMyDigits(n: number): number {
    if (n > 1000) {
      return Math.round(n);
    } else if (n > 100) {
      return Math.round(n);
    } else if (n > 10) {
      return Math.floor(Math.round(n * 10)) / 10;
    } else if (n > 1) {
      return Math.floor(Math.round(n * 100)) / 100;
    } else if (n > 0.1) {
      return Math.floor(Math.round(n * 1000)) / 1000;
    } else if (n > 0.01) {
      return Math.floor(Math.round(n * 10000)) / 10000;
    } else if (n > 0.001) {
      return Math.floor(Math.round(n * 100000)) / 100000;
    } else if (n > 0.0001) {
      return Math.floor(Math.round(n * 1000000)) / 1000000;
    } else if (n > 0.00001) {
      return Math.floor(Math.round(n * 10000000)) / 10000000;
    } else {
      return n;
    }
  }

  public roundMyNumbers(n: number): number {
    if (n > 1000) {
      return Math.floor(n / 1000) * 1000;
    } else if (n > 100) {
      return Math.floor(n / 100) * 100;
    } else if (n > 10) {
      return Math.floor(n / 10) * 10;
    } else if (n > 1) {
      return Math.floor(n / 1) * 1;
    } else if (n > 0.1) {
      return Math.floor(n * 10) / 10;
    } else if (n > 0.01) {
      return Math.floor(n * 100) / 100;
    } else if (n > 0.001) {
      return Math.floor(n * 1000) / 1000;
    } else if (n > 0.0001) {
      return Math.floor(n * 10000) / 10000;
    } else if (n > 0.00001) {
      return Math.floor(n * 100000) / 100000;
    } else {
      return n;
    }
  }

  setLegend(legendType) {
    if (this.map != undefined && legendType == undefined) {
      this.map.removeControl(this.mapLegend);
      return;
    }
    if (this.map == undefined) {
      return;
    }

    this.mapLegend.onAdd = (map) => {
      let direction: string;
      if (legendInverted.indexOf(legendType) == -1) {
        direction = "lowToHigh";
      } else {
        direction = "highToLow";
      }

      if (
        this.maxKeyValues["maxValue_" + legendType] == undefined ||
        this.maxKeyValues["maxValue_" + legendType] == 0
      ) {
        this.toastrService.info(
          "Geojson not displayed because all values are undefined or zero!",
          "Geojson not displayed"
        );
        let div = L.DomUtil.create("div");
        div.innerHTML += `<p style='background:white; display: inline; font-size:17px; padding: 0 7px; background: #ff3d71'>Values for ${legendType} kpi are undefined or zero!</p>`;
        return div;
      }

      var div = L.DomUtil.create("div");

      this.setInputsLegendContainerColors(direction);

      if (direction == "lowToHigh") {
        div.innerHTML +=
          "<p style='background:white; display: inline; font-size:17px'> <span style='color:" +
          MapColors.getColorForLegend(0) +
          "'>&#9632</span> <" +
          this.brinkValues[legendType][0] +
          "</p>";
        for (let i = 1; i < 6; i++) {
          div.innerHTML +=
            "<p style='background:white; display: inline; font-size:17px'> <span style='color:" +
            MapColors.getColorForLegend(i) +
            "'>&#9632</span> " +
            this.brinkValues[legendType][i - 1] +
            (this.brinkValues[legendType][i]
              ? "&ndash;" + this.brinkValues[legendType][i] + "</p>"
              : "+</p>");

          this.newBrinkValuesForm
            .get(`legend_${i - 1}`)
            .setValue(`${this.brinkValues[legendType][i - 1]}`);
        }
      } else {
        div.innerHTML +=
          "<p style='background:white; display: inline; font-size:17px'> <span style='color:" +
          MapColors.getColorForLegendHighToLow(0) +
          "'>&#9632</span> <" +
          this.brinkValues[legendType][0] +
          "</p>";

        for (let i = 0; i < 5; i++) {
          div.innerHTML +=
            "<p style='background:white; display: inline; font-size:17px'> <span style='color:" +
            MapColors.getColorForLegendHighToLow(i + 1) +
            "'>&#9632</span> " +
            this.brinkValues[legendType][i] +
            (this.brinkValues[legendType][i + 1]
              ? "&ndash;" + this.brinkValues[legendType][i + 1] + "</p>"
              : "+</p>");

          this.newBrinkValuesForm
            .get(`legend_${i}`)
            .setValue(`${this.brinkValues[legendType][i]}`);
        }
      }

      return div;
    };
    this.mapLegend.addTo(this.map);
  }

  grayscaleChange() {
    const el = <HTMLElement>(
      document.querySelectorAll(".leaflet-pane.leaflet-tile-pane")[0]
    );
    el.style.filter = "grayscale(" + (100 - this.grayscale) + "%)";
  }

  setLoadGeojsons() {
    if (this.selectedSimulation != undefined) {
      if (this.selectedSimulation["city"]["cityId"] == "amsterdam") {
        this.loadGeojsons = [
          {
            response: undefined,
            buttonValue: "bikeability",
            buttonLabel: "bikeability",
          },
          {
            response: undefined,
            buttonValue: "bikeIntensity",
            buttonLabel: "bike Intensity",
          },
          {
            response: undefined,
            buttonValue: "bikeSafety",
            buttonLabel: "bike Safety",
          },
          {
            response: undefined,
            buttonValue: "pollution",
            buttonLabel: "pollution",
          },
        ];
      } else if (this.selectedSimulation["city"]["cityId"] == "bilbao") {
        this.loadGeojsons = [
          {
            response: undefined,
            buttonValue: "bikeability",
            buttonLabel: "bikeability",
          },
          {
            response: undefined,
            buttonValue: "bikeIntensity",
            buttonLabel: "bike Intensity",
          },
          {
            response: undefined,
            buttonValue: "bikeSafety",
            buttonLabel: "bike Safety",
          },
          {
            response: undefined,
            buttonValue: "pollution",
            buttonLabel: "pollution",
          },
        ];
      } else if (this.selectedSimulation["city"]["cityId"] == "helsinki") {
        this.loadGeojsons = [
          {
            response: undefined,
            buttonValue: "congestionsAndBottlenecks",
            buttonLabel: "congestions And Bottlenecks",
          },
          {
            response: undefined,
            buttonValue: "harbourAreaTrafficFlow",
            buttonLabel: "harbour Area Traffic Flow",
          },
          {
            response: undefined,
            buttonValue: "accousticPollution",
            buttonLabel: "accoustic Pollution",
          },
          {
            response: undefined,
            buttonValue: "pollution",
            buttonLabel: "pollution",
          },
        ];
      } else if (this.selectedSimulation["city"]["cityId"] == "messina") {
        this.loadGeojsons = [
          {
            response: undefined,
            buttonValue: "bikeability",
            buttonLabel: "bikeability",
          },
          {
            response: undefined,
            buttonValue: "bikeIntensity",
            buttonLabel: "bike Intensity",
          },
          {
            response: undefined,
            buttonValue: "bikeSafety",
            buttonLabel: "bike Safety",
          },
          {
            response: undefined,
            buttonValue: "pollution",
            buttonLabel: "pollution",
          },
        ];
      } else {
        this.toastrService.show("Unexpected value for city name.", "Error", {
          status: "danger",
        });
      }
    }
  }

  openOverviewOfIndicators(dialog: TemplateRef<any>) {
    this.dialogService.open(dialog, {
      context: this.selectedSimulation["city"]["cityId"],
    });
  }

  resetLayersAndButton() {
    if (this.map != undefined) {
      this.map.eachLayer((layer) => {
        if (layer["feature"] != undefined) {
          this.map.removeLayer(layer);
        }
      });

      if (this.radioButtonLayersControl)
        this.radioButtonLayersControl.remove(this.map);
    }
    this.setLegend(undefined);
  }

  ngOnChanges() {
    this.resetLayersAndButton();
    this.selectedGeojson = "";

    Object.entries(this.loadGeojsons).forEach((entry) => {
      entry[1]["response"] = undefined;
    });

    this.setLoadGeojsons();
  }

  setMapPositionAndZoom(res: Object) {
    this.map.setView(
      new L.LatLng(res["coordinates"]["lat"], res["coordinates"]["lon"]),
      res["zoom"]
    );
  }

  setInputsLegendContainerColors(direction: string): void {
    if (direction == "lowToHigh") {
      const legendLabels = document.querySelectorAll(
        "#inputsLegendContainer > form > div > div > label"
      );
      for (let i = 0; i < legendLabels.length; i++) {
        (legendLabels[i] as HTMLElement).style.backgroundColor =
          MapColors.colorsAll[legendLabels.length - i - 1];
      }
      const inputsLegend = document.querySelectorAll(
        "#inputsLegendContainer > form > div > div > input"
      );
      for (let i = 0; i < inputsLegend.length; i++) {
        (inputsLegend[i] as HTMLElement).style.backgroundColor =
          MapColors.colorsAll[inputsLegend.length - i];
      }
    } else {
      const legendLabels = document.querySelectorAll(
        "#inputsLegendContainer > form > div > div > label"
      );
      for (let i = 0; i < legendLabels.length; i++) {
        (legendLabels[i] as HTMLElement).style.backgroundColor =
          MapColors.colorsAll[i];
      }
      const inputsLegend = document.querySelectorAll(
        "#inputsLegendContainer > form > div > div > input"
      );
      for (let i = 0; i < inputsLegend.length; i++) {
        (inputsLegend[i] as HTMLElement).style.backgroundColor =
          MapColors.colorsAll[i];
      }
    }
  }
}
