import {
  animate,
  state,
  style,
  transition,
  trigger,
  AnimationEvent,
} from "@angular/animations";
import { Component, OnInit } from "@angular/core";
import { AbstractControl, FormBuilder, Validators } from "@angular/forms";
import { RxStompService } from "@stomp/ng2-stompjs";
import { socketStatusAnimation } from "../utils/animations/socket-status.animation";
import { Network } from "../utils/data/network";
import { Scenario } from "../utils/data/scenario";
import { Simulation } from "../utils/data/simulation";
import { CityService } from "../utils/services/city.service";
import { ScenarioService } from "../utils/services/scenario.service";
import { SimulationService } from "../utils/services/simulation.service";

@Component({
  selector: "ngx-scenarios",
  templateUrl: "./scenarios.component.html",
  styleUrls: ["./scenarios.component.scss"],
  animations: [
    trigger("expansionIndicator", [
      state(
        "expanded",
        style({
          transform: "rotate(180deg)",
        })
      ),
      transition("collapsed => expanded", animate("100ms ease-in")),
      transition("expanded => collapsed", animate("100ms ease-out")),
    ]),
    trigger("accordionItemBody", [
      state(
        "collapsed",
        style({
          overflow: "hidden",
          visibility: "hidden",
          height: 0,
        })
      ),
      state(
        "expanded",
        style({
          overflow: "hidden",
          visibility: "visible",
        })
      ),
      transition("collapsed => expanded", animate("100ms ease-in")),
      transition("expanded => collapsed", animate("100ms ease-out")),
    ]),
    socketStatusAnimation,
  ],
})
export class ScenariosComponent implements OnInit {
  scenarioForm = this.fb.group({
    name: ["", Validators.required],
    description: [""],
    networkId: ["", Validators.required],
  });

  scenarios: Scenario[];
  simulations: Simulation[];
  networks: Network[];

  selectedScenarioIndex = 0;

  constructor(
    private cityService: CityService,
    private scenarioService: ScenarioService,
    private simulationService: SimulationService,
    private fb: FormBuilder,
    private rxStompService: RxStompService
  ) {}

  ngOnInit() {
    this.cityService.getSelectedCity().subscribe((city) => {
      this.cityService.getCityScenarios(city.cityId).subscribe((scenarios) => {
        this.scenarios = scenarios;
        console.log(
          "%c scenarios : ",
          "background: cornflowerblue; font-weight: bold; color: black;",
          this.scenarios
        );
        if (this.scenarios.length > 0) this.getSimulations();
      });

      this.cityService
        .getCityNetworks(city.cityId)
        .subscribe((networks) => (this.networks = networks));
    });

    this.rxStompService
      .watch("/topic/simulationRunStatus")
      .subscribe((updatedSimulationFrame) => {
        let updatedSimulation = JSON.parse(updatedSimulationFrame.body);
        let index = this.simulations.findIndex(
          (simulation) => simulation.id == updatedSimulation.id
        );
        if (index == -1) {
          this.simulations.push(Simulation.deserialize(updatedSimulation));
        } else {
          this.simulations[index] = Simulation.deserialize(updatedSimulation);
          this.simulations[index].statusWSMessage =
            updatedSimulationFrame.headers.statusWSMessage;
          this.simulations[index].animationState =
            this.simulations[index].animationState == "hidden"
              ? "show"
              : "replaceHide";
        }
      });
  }

  getSimulations() {
    this.scenarioService
      .getScenarioSimulations(this.selectedScenario.id)
      .subscribe((simulations) => (this.simulations = simulations));
  }

  handleSelectScenarioClick(i: number) {
    this.selectedScenarioIndex = i;
    this.getSimulations();
  }

  onSubmit() {
    let simulationNew = Simulation.deserialize(this.scenarioForm.value);
    simulationNew.scenarioId = this.selectedScenario.id;
    this.simulationService
      .createSimulation(simulationNew)
      .subscribe((simulation) => this.simulations.push(simulation));
  }

  runSimulationClickHandler(simulation: Simulation) {
    this.simulationService.runSimulation(simulation).subscribe((simulation) => {
      let index = this.simulations.findIndex(
        (simulationOld) => simulationOld.id == simulation.id
      );
      this.simulations[index] = simulation;
    });
  }

  onAnimationEventDone(event: AnimationEvent, index: number) {
    if (
      event.toState == "show" &&
      this.simulations[index].animationState == "show"
    )
      this.simulations[index].animationState = "shown";
    else if (
      event.toState == "shown" &&
      this.simulations[index].animationState == "shown"
    )
      this.simulations[index].animationState = "hide";
    else if (
      event.toState == "hide" &&
      this.simulations[index].animationState == "hide"
    )
      this.simulations[index].animationState = "hidden";
    else if (
      event.toState == "replaceHide" &&
      this.simulations[index].animationState == "replaceHide"
    )
      this.simulations[index].animationState = "replaceShow";
    else if (
      event.toState == "replaceShow" &&
      this.simulations[index].animationState == "replaceShow"
    )
      this.simulations[index].animationState = "shown";
  }

  onAnimationEventStart(event: AnimationEvent, index: number) {
    if (event.toState == "show" || event.toState == "replaceShow") {
      (event.element as HTMLElement).innerHTML =
        this.simulations[index].statusWSMessage;
    }
  }

  get selectedScenario() {
    return this.scenarios[this.selectedScenarioIndex];
  }

  get name() {
    return this.scenarioForm.get("name");
  }

  get description() {
    return this.scenarioForm.get("description");
  }

  get networkId() {
    return this.scenarioForm.get("networkId");
  }

  checkValid(control: AbstractControl) {
    return control.invalid && (control.dirty || control.touched);
  }

  // Might move to separate component
  selectScenarioBoxState = "collapsed";
}
