import { AnimationEvent } from "@angular/animations";
import { Component, OnInit } from "@angular/core";
import { AbstractControl, FormBuilder, Validators } from "@angular/forms";
import { RxStompService } from "@stomp/ng2-stompjs";
import { socketStatusAnimation } from "../utils/animations/socket-status.animation";
import { City } from "../utils/data/city";
import { Network } from "../utils/data/network";
import { CityService } from "../utils/services/city.service";
import { NetworkService } from "../utils/services/network.service";
import { ScenarioService } from "../utils/services/scenario.service";
import { SimulationService } from "../utils/services/simulation.service";
import { Scenario } from "../utils/data/scenario";
import { Simulation } from "../utils/data/simulation";
import { NbToastrService, NbDialogRef } from "@nebular/theme";

@Component({
  selector: "ngx-simulation-creation",
  templateUrl: "./simulation-creation.component.html",
  styleUrls: ["./simulation-creation.component.scss"],
  animations: [socketStatusAnimation],
})
export class SimulationCreationComponent implements OnInit {
  networks: Network[] = [];
  city: City;
  scenarios: Scenario[] = [];
  simulations: Simulation[] = [];
  addSimulationForm = this.fb.group({
    name: ["Sample name", Validators.required],
    description: [""],
    selectedScenario: {
      value: undefined,
      disabled: false,
    },
    selectedNetwork: {
      value: undefined,
      disabled: false,
    },
  });
  selectedScenario;
  selectedNetwork;

  constructor(
    private networkService: NetworkService,
    private cityService: CityService,
    private fb: FormBuilder,
    private scenarioService: ScenarioService,
    private simulationService: SimulationService,
    private rxStompService: RxStompService,
    private toastrService: NbToastrService,
    protected dialogRef: NbDialogRef<any>
  ) {}

  ngOnInit(): void {
    this.cityService.getSelectedCity().subscribe((city) => {
      this.city = city;
    });

    this.rxStompService
      .watch("/topic/networkProcessingStatus")
      .subscribe((updatedNetworkFrame) => {
        let updatedNetwork = JSON.parse(updatedNetworkFrame.body);
        let index = this.networks.findIndex(
          (network) => network.id == updatedNetwork.id
        );
        if (index == -1) {
          this.networks.push(Network.deserialize(updatedNetwork));
        } else {
          this.networks[index] = Network.deserialize(updatedNetwork);
          this.networks[index].conversionWSMessage =
            updatedNetworkFrame.headers.conversionWSMessage;
          this.networks[index].animationState =
            this.networks[index].animationState == "hidden"
              ? "show"
              : "replaceHide";
        }
      });

    this.scenarioService.getScenarios().subscribe(
      (data) => {
        this.scenarios = data;
        this.selectedScenario = data[0];
      },
      (error) => {
        this.toastrService.show(
          `${error["message"]}`,
          `${error["error"]["error"]}`,
          {
            status: "danger",
          }
        );
      }
    );

    this.networkService.getNetworks().subscribe(
      (data) => {
        this.networks = data;
        this.selectedNetwork = data[0];
      },
      (error) => {
        console.log(
          "🚀 ~ file: simulation-creation.component.ts ~ line 105 ~ SimulationCreationComponent ~ this.networkService.getNetworks ~ error",
          error
        );
        this.toastrService.show(
          `${error["message"]}`,
          `${error["error"]["error"]}`,
          {
            status: "danger",
          }
        );
      }
    );
  }

  addSimulationSubmit() {
    let sim = new Simulation();
    sim.name = this.addSimulationForm.controls.name.value;
    sim.description = this.addSimulationForm.controls.description.value;
    sim.networkId = this.selectedScenario.id;
    sim.scenarioId = this.selectedNetwork.id;
    this.simulationService.createSimulation(sim).subscribe(
      (res) => {
        this.toastrService.show(`${res["message"]}`, "Success", {
          status: "success",
        });
      },
      (error: object) => {
        this.toastrService.show(
          `${error["message"]}`,
          `${error["error"]["error"]}`,
          {
            status: "danger",
          }
        );
      }
    );
  }

  onSelectNetwork(id: number) {
    this.networkService.getNetwork(id).subscribe((res) => {
      console.log(
        "🚀 ~ file: simulation-creation.component.ts ~ line 155 ~ SimulationCreationComponent ~ this.networkService.getNetwork ~ res",
        res.name,
        res.id,
        res.description,
        res.doneOsmProcessing,
        res.doneMATSimProcessing,
        res.conversionError
      );
    });
  }

  onSelectScenarioChange(evt) {
    this.addSimulationForm.controls.selectedNetwork.enable();
  }

  onAnimationEventDone(event: AnimationEvent, index: number) {
    if (
      event.toState == "show" &&
      this.networks[index].animationState == "show"
    )
      this.networks[index].animationState = "shown";
    else if (
      event.toState == "shown" &&
      this.networks[index].animationState == "shown"
    )
      this.networks[index].animationState = "hide";
    else if (
      event.toState == "hide" &&
      this.networks[index].animationState == "hide"
    )
      this.networks[index].animationState = "hidden";
    else if (
      event.toState == "replaceHide" &&
      this.networks[index].animationState == "replaceHide"
    )
      this.networks[index].animationState = "replaceShow";
    else if (
      event.toState == "replaceShow" &&
      this.networks[index].animationState == "replaceShow"
    )
      this.networks[index].animationState = "shown";
  }

  onAnimationEventStart(event: AnimationEvent, index: number) {
    if (event.toState == "show" || event.toState == "replaceShow") {
      (event.element as HTMLElement).innerHTML =
        this.networks[index].conversionWSMessage;
    }
  }

  //  Boilerplate code for easier form elements access
  get name() {
    return this.addSimulationForm.get("name");
  }

  get description() {
    return this.addSimulationForm.get("description");
  }

  get bounds() {
    return this.addSimulationForm.get("bounds");
  }

  checkValid(control: AbstractControl) {
    return control.invalid && (control.dirty || control.touched);
  }

  onDiscard() {
    this.dialogRef.close();
  }
}
