import { HttpClient } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { Observable, throwError } from "rxjs";
import { map, tap, catchError } from "rxjs/operators";
import { Simulation } from "../data/simulation";
import { tapDebugApi } from "./tap-debug";
import { ConfigService } from "@ngx-config/core";

@Injectable({
  providedIn: "root",
})
export class SimulationService {
  private ts_simulation_url: any;
  private ts_url: any;
  private storage_url: any;
  private dssUrl: any;

  constructor(
    private httpClient: HttpClient,
    private configService: ConfigService
  ) {
    this.ts_simulation_url = `${configService.getSettings(
      "traffic_simulation_base_url"
    )}/simulation`;
    this.ts_url = `${configService.getSettings("traffic_simulation_base_url")}`;
    this.storage_url = `${configService.getSettings(
      "traffic_simulation_storage_url"
    )}`;
    this.dssUrl = `${configService.getSettings("traffic_simulation_dss_url")}`;
  }

  createSimulation(simulation: Simulation): Observable<Simulation> {
    return this.httpClient.post(this.ts_simulation_url, simulation).pipe(
      map((simulation) => Simulation.deserialize(simulation)),
      tapDebugApi
    );
  }

  runSimulation(simulation: Simulation): Observable<Simulation> {
    return this.httpClient
      .get(`${this.ts_simulation_url}/${simulation.id}/run`, {})
      .pipe(
        map((simulation) => Simulation.deserialize(simulation)),
        tapDebugApi
      );
  }

  getSimulations(city_id: string): Observable<string> {
    return this.httpClient
      .get(`${this.ts_url}/city/${city_id}/simulations`, {})
      .pipe(
        map((simulation) => Simulation.deserialize(simulation)),
        tapDebugApi
      );
  }

  getSimulationsAll(): Observable<string> {
    return this.httpClient.get(`${this.ts_simulation_url}`, {}).pipe(
      map((simulation) => Simulation.deserialize(simulation)),
      tapDebugApi
    );
  }

  postEvaluated(arrayToBeSent: any): Observable<any> {
    return this.httpClient.post(`${this.storage_url}/storage/dss/evaluated`, {
      ids: arrayToBeSent,
    });
  }

  getPreprocessData(city_id: string): Observable<any> {
    return this.httpClient
      .get(`${this.ts_url}/scripts/city/${city_id}/preprocess_data`, {})
      .pipe(
        map((res) => {
          return res;
        }),
        catchError((error) => {
          return throwError(error);
        }),
        tapDebugApi
      );
  }

  getGeneratePopulation(city_id: string): Observable<any> {
    return this.httpClient
      .get(`${this.ts_url}/scripts/city/${city_id}/generate_population`, {})
      .pipe(
        map((res) => {
          return res;
        }),
        catchError((error) => {
          return throwError(error);
        }),
        tapDebugApi
      );
  }

  postTravelDemand(
    city_id: string,
    name: string,
    networkId: string,
    simulationId: number
  ): Observable<any> {
    return this.httpClient
      .post(`${this.ts_url}/scripts/city/${city_id}/travel_demand`, {
        name: name,
        networkId: networkId,
        simulationId: simulationId,
      })
      .pipe(
        map((res) => {
          return res;
        })
      );
  }

  getCalculateKPIs(cityId: string, id: number): Observable<any> {
    return this.httpClient
      .get(`${this.dssUrl}/dss/kpis/${cityId}/${id}`, {})
      .pipe(
        map((res) => {
          return res;
        }),
        catchError((error) => {
          return throwError(error);
        }),
        tapDebugApi
      );
  }

  postRunDss(
    [baselineId, compareId]: [number, number],
    cityId: string
  ): Observable<any> {
    return this.httpClient
      .post(`${this.dssUrl}/dss/kpi_eval/${cityId}`, {
        baseline: baselineId,
        compare: compareId,
      })
      .pipe(
        map((res) => {
          return res;
        }),
        catchError((error) => {
          return throwError(error);
        }),
        tapDebugApi
      );
  }

  postCalibrationPreprocess(
    dateFrom: Date,
    dateTo: Date,
    networkId: number
  ): Observable<any> {
    let formData = new FormData();
    formData.append("date_from", this.formatDateToOurSpec(dateFrom).toString());
    formData.append("date_to", this.formatDateToOurSpec(dateTo).toString());
    formData.append("network_id", networkId.toString());

    return this.httpClient
      .post(`${this.dssUrl}/dss/calibration_preprocess`, formData)
      .pipe(
        map((res) => {
          return res;
        }),
        catchError((error) => {
          return throwError(error);
        }),
        tapDebugApi
      );
  }

  getNetworkFromSimulation(id: number): Observable<any> {
    return this.httpClient
      .get(`${this.ts_simulation_url}/${id}/network`, {})
      .pipe(
        map((res) => {
          return res;
        }),
        catchError((error) => {
          return throwError(error);
        }),
        tapDebugApi
      );
  }

  formatDateToOurSpec(date: Date): String {
    return (
      date.getFullYear() +
      "-" +
      String(date.getMonth() + 1).padStart(2, "0") +
      "-" +
      String(date.getDate()).padStart(2, "0")
    );
  }

  getTravelDemandModel(networkId: string): Observable<any> {
    return this.httpClient
      .get(`${this.storage_url}/storage/plans/${networkId}`)
      .pipe(
        map((res) => {
          return res;
        }),
        tapDebugApi
      );
  }

  getGeojson(id: string, selectedKpi: string): Observable<JSON> {
    return this.httpClient
      .get(`${this.dssUrl}/dss/geojson/${id}/${selectedKpi}`)
      .pipe(
        map((res) => {
          return res;
        }),
        tapDebugApi
      );
  }
}
