import { AfterViewInit, Component, OnInit } from "@angular/core";
import * as L from "leaflet";
import { SimulationService } from "../../utils/services/simulation.service";
import "rxjs/add/observable/forkJoin";
import "rxjs/add/operator/first";
import * as MapColors from "../../utils/data/map-colors";
import { brink_values } from "./brink_values";
import { NbToastrService } from "@nebular/theme";

@Component({
  selector: "ngx-visualizations-geojson",
  templateUrl: "./visualizations-geojson.component.html",
  styleUrls: ["./visualizations-geojson.component.scss"],
})
export class VisualizationsGeojsonComponent implements OnInit, AfterViewInit {
  public map: L.Map;
  options = {
    layers: [
      L.tileLayer("http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png", {
        maxZoom: 18,
        attribution: "...",
      }),
    ],
    zoom: 11,
    center: L.latLng({ lat: 43.263, lng: -2.935 }),
  };
  mapLegend = new L.Control({ position: "bottomright" });
  radioButtonLayersControl: any;
  checkedRadioButton: string = "";
  geoJsonStyleWeight: number = 5;
  geoJsonStyleOpacity: number = 0.8;

  constructor(
    private simulationService: SimulationService,
    private toastrService: NbToastrService
  ) {}

  ngOnInit() {}

  ngAfterViewInit(): void {}

  onMapReady(map: L.Map) {
    this.map = map;
    setTimeout(() => {
      this.map.invalidateSize();
    }, 50);
    this.onLoad();
  }

  onLoad() {
    if (this.radioButtonLayersControl)
      this.radioButtonLayersControl.remove(this.map);

    console.log(
      "%c" + "doing something for hardcoded id = 2",
      "font-family:Comic Sans MS; font-weight:bold;color:black; background: linear-gradient(#f00, yellow); border-radius: 5px; padding-left: 5px; padding-right: 5px"
    );
    // # TODO hardcoded variable
    this.simulationService.getGeojson("2").subscribe((res) => {
      if (
        res["message"] ==
        "The visualization is not prepared yet, try again in a few minutes."
      ) {
        this.toastrService.show(
          "The visualization is not prepared yet, try again in a few minutes.",
          "Warning",
          {
            status: "warning",
          }
        );
        return;
      }
      const keys = this.extractKeys(res);
      const propertyGeojsons = this.populatePropertyGeojsons(res, keys);
      let obj = {};

      for (let i = 0; i < propertyGeojsons.length; i++) {
        if (
          propertyGeojsons[i].myKey == "link_id" ||
          propertyGeojsons[i].myKey == "modes" ||
          propertyGeojsons[i].myKey == "internal_travel_by_mode"
        )
          continue;
        obj[`${keys[i]}`] = propertyGeojsons[i];
      }
      this.radioButtonLayersControl = new L.Control.Layers(obj, null, {
        collapsed: false,
      });
      this.map.addControl(this.radioButtonLayersControl);
      this.addListenerToRadioButtonLayersControl();
    });
  }

  // LEGEND
  setLegend(legendType) {
    this.mapLegend.onAdd = (map) => {
      var div = L.DomUtil.create("div");

      if (
        legendType == "PM" ||
        legendType == "CO2_rep" ||
        legendType == "CO" ||
        legendType == "CO2_TOTAL" ||
        legendType == "HC" ||
        legendType == "NOx" ||
        legendType == "dailyInternalBikeTravels" ||
        legendType == "pedestrianTravelTime"
      ) {
        let grades = brink_values["brink_" + legendType];
        div.innerHTML +=
          "<p style='background:white; display: inline; font-size:17px'> <span style='color:" +
          MapColors.getColorUniversal(0, brink_values["brink_" + legendType]) +
          "'>&#9632</span> <" +
          grades[1] +
          "</p>";
        for (let i = 1; i < 6; i++) {
          div.innerHTML +=
            "<p style='background:white; display: inline; font-size:17px'> <span style='color:" +
            MapColors.getColorUniversal(
              i,
              brink_values["brink_" + legendType]
            ) +
            "'>&#9632</span> " +
            grades[i] +
            (grades[i + 1] ? "&ndash;" + grades[i + 1] + "</p>" : "+</p>");
        }
      } else if (legendType == "capacity") {
        let grades = brink_values["brink_" + legendType];
        for (let i = 5; i > 0; i--) {
          div.innerHTML +=
            "<p style='background:white; display: inline; font-size:17px'> <span style='color:" +
            MapColors.getColorUniversal(
              5 - i,
              brink_values["brink_" + legendType]
            ) +
            "'>&#9632</span> " +
            grades[i] +
            (grades[i + 1] ? "&ndash;" + grades[i + 1] + "</p>" : "+</p>");
        }
        div.innerHTML +=
          "<p style='background:white; display: inline; font-size:17px'> <span style='color:" +
          MapColors.getColorUniversal(5, brink_values["brink_" + legendType]) +
          "'>&#9632</span> <" +
          grades[1] +
          "</p>";
      } else {
        let grades = [0, 1, 10, 100, 1000, 10000];
        // loop through our density intervals and generate a label with a colored square for each interval
        for (let i = 0; i < 6; i++) {
          div.innerHTML +=
            "<p style='background:white; display: inline; font-size:17px'> <span style='color:" +
            MapColors.getColorUniversal(i, brink_values["brink_CO2_rep"]) +
            "'>&#9632</span> " +
            grades[i] +
            (grades[i + 1] ? "&ndash;" + grades[i + 1] + "</p>" : "+</p>");
        }
      }

      return div;
    };
    this.mapLegend.addTo(this.map);
  }

  private highlightFeature(e, feature) {
    const layer = e.target;
    let text = "";
    if (feature["capacity"]) {
      if ("capacity" == this.checkedRadioButton) {
        text += `<u><strong>capacity:</strong> ${feature["capacity"]}</u><br>`;
      } else {
        text += `<strong>capacity:</strong> ${feature["capacity"]}<br>`;
      }
    }

    for (let p in feature["cityWide"]["pollution"]) {
      if (p == this.checkedRadioButton) {
        text += `<u><strong>${p}:</strong> ${feature["cityWide"]["pollution"][p]}</u><br>`;
      } else {
        text += `<strong>${p}:</strong> ${feature["cityWide"]["pollution"][p]}<br>`;
      }
    }
    for (let p in feature["cityWide"]["traffic"]) {
      if (p == this.checkedRadioButton) {
        text += `<u><strong>${p}:</strong> ${feature["cityWide"]["traffic"][p]}</u><br>`;
      } else {
        text += `<strong>${p}:</strong> ${feature["cityWide"]["traffic"][p]}<br>`;
      }
    }

    if (!text.length)
      text = "<span style='color: lightslategray';>No data available.</span>";
    layer.bindPopup(text);
    layer.setStyle({
      weight: 10,
      opacity: 1.0,
    });
  }

  private resetFeature(e, feature) {
    const layer = e.target;

    layer.setStyle({
      weight: this.geoJsonStyleWeight,
      opacity: this.geoJsonStyleOpacity,
    });
  }

  public extractKeys(jsonFile) {
    let allKeys = [];
    for (let f = 0; f < jsonFile["features"].length && f < 3498; f++) {
      let iterator;
      if (jsonFile["features"][f]["properties"]["capacity"]) {
        if (!allKeys.includes("capacity")) {
          allKeys.push("capacity");
        }
      }

      if (jsonFile["features"][f]["properties"]["cityWide"]["pollution"]) {
        iterator = Object.keys(
          jsonFile["features"][f]["properties"]["cityWide"]["pollution"]
        );
        for (let g = 0; g < iterator.length; g++) {
          if (!allKeys.includes(iterator[g])) {
            allKeys.push(iterator[g]);
          }
        }
      }
      if (jsonFile["features"][f]["properties"]["cityWide"]["traffic"]) {
        iterator = Object.keys(
          jsonFile["features"][f]["properties"]["cityWide"]["traffic"]
        );
        for (let g = 0; g < iterator.length; g++) {
          if (!allKeys.includes(iterator[g])) {
            allKeys.push(iterator[g]);
          }
        }
      }
    }
    return allKeys;
  }

  public populatePropertyGeojsons(jsonFile, keys) {
    let res = [];
    for (let f = 0; f < keys.length; f++) {
      let tmp;

      if (keys[f] == "capacity") {
        tmp = L.geoJSON(<any>jsonFile, {
          style: (feature) => ({
            weight: this.geoJsonStyleWeight,
            color: MapColors.getColorCapacity(feature.properties.capacity),
            opacity: this.geoJsonStyleOpacity,
          }),
          onEachFeature: (feature, layer) =>
            layer.on({
              mouseover: (e) => this.highlightFeature(e, feature.properties),
              mouseout: (e) => this.resetFeature(e, feature.properties),
            }),
        });
      } else if (
        keys[f] == "dailyInternalBikeTravels" ||
        keys[f] == "pedestrianTravelTime"
      ) {
        tmp = L.geoJSON(<any>jsonFile, {
          style: (feature) => ({
            weight: this.geoJsonStyleWeight,
            color: MapColors.getColor(
              feature.properties.cityWide.traffic,
              keys[f]
            ),
            opacity: this.geoJsonStyleOpacity,
          }),
          onEachFeature: (feature, layer) =>
            layer.on({
              mouseover: (e) => this.highlightFeature(e, feature.properties),
              mouseout: (e) => this.resetFeature(e, feature.properties),
            }),
        });
      } else {
        tmp = L.geoJSON(<any>jsonFile, {
          style: (feature) => ({
            weight: this.geoJsonStyleWeight,
            color: MapColors.getColor(
              feature.properties.cityWide.pollution,
              keys[f]
            ),
            opacity: this.geoJsonStyleOpacity,
          }),
          onEachFeature: (feature, layer) =>
            layer.on({
              mouseover: (e) => this.highlightFeature(e, feature.properties),
              mouseout: (e) => this.resetFeature(e, feature.properties),
            }),
        });
      }

      tmp["myKey"] = keys[f];
      res.push(tmp);
    }
    return res;
  }

  public addListenerToRadioButtonLayersControl() {
    this.map.on("baselayerchange", (e) => {
      if (e["name"] == "CO") {
        this.setLegend("CO");
        this.checkedRadioButton = "CO";
      } else if (e["name"] == "CO2_TOTAL") {
        this.setLegend("CO2_TOTAL");
        this.checkedRadioButton = "CO2_TOTAL";
      } else if (e["name"] == "HC") {
        this.setLegend("HC");
        this.checkedRadioButton = "HC";
      } else if (e["name"] == "NOx") {
        this.setLegend("NOx");
        this.checkedRadioButton = "NOx";
      } else if (e["name"] == "PM") {
        this.setLegend("PM");
        this.checkedRadioButton = "PM";
      } else if (e["name"] == "CO2_rep") {
        this.setLegend("CO2_rep");
        this.checkedRadioButton = "CO2_rep";
      } else if (e["name"] == "dailyInternalBikeTravels") {
        this.setLegend("dailyInternalBikeTravels");
        this.checkedRadioButton = "dailyInternalBikeTravels";
      } else if (e["name"] == "pedestrianTravelTime") {
        this.setLegend("pedestrianTravelTime");
        this.checkedRadioButton = "pedestrianTravelTime";
      } else if (e["name"] == "capacity") {
        this.setLegend("capacity");
        this.checkedRadioButton = "capacity";
      } else {
        this.setLegend(undefined);
        this.checkedRadioButton = undefined;
      }
    });
  }
}
